# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import sys
import logging
import bcrypt
from bson import ObjectId
from bson.errors import InvalidId
import superdesk
from superdesk.utils import gen_password
from superdesk.auth_server.scopes import allowed_scopes


logger = logging.getLogger(__name__)


class AuthServerClientsResource(superdesk.Resource):

    schema = {
        "name": {
            "type": "string",
            "required": True,
            "unique": True,
        },
        "password": {"type": "string", "required": True},
        "scope": {"type": "list", "allowed": list(allowed_scopes), "required": True},
    }


class AuthServerClientsService(superdesk.Service):
    """Service to handle authorization server clients"""


class CommonClient:
    """Common method for client commands"""

    def validate_scope(self, scope):
        """Return list of unique and valid scopes, end command execution else"""
        scope = set(scope)
        if not scope.issubset(allowed_scopes):
            self.parser.error(
                "invalid scopes: {invalid_scopes}\nvalid scopes are: {allowed_scopes}".format(
                    invalid_scopes=", ".join(scope - allowed_scopes), allowed_scopes=", ".join(allowed_scopes)
                )
            )
        return list(scope)


class RegisterClient(superdesk.Command, CommonClient):
    """Register a client to authentication server

    A client name is needed, and an id and password will be generated and displayed once
    the client is registered.

    The client will be allowed to access the given scopes using the ``--scope`` argument
    (this argument may be used several times).

    Example:
    Register a new client with name ``my Superdesk client`` and allowed to reach items in archive and get users::

        $ python manage.py auth_server:register_client "my Superdesk client" -s ARCHIVE_READ -s USERS_READ

    """

    option_list = [
        superdesk.Option(
            "--client-id", "-i", dest="client_id", help="ObjectId compatible client id (keep empty to generate one)"
        ),
        superdesk.Option("--password", "-p", nargs="?", const="", help="client password (keep empty to generate it)"),
        superdesk.Option(
            "--scope",
            "-s",
            action="append",
            default=[],
            help="scopes allowed (one or more of {allowed_scopes})".format(allowed_scopes=", ".join(allowed_scopes)),
        ),
        # empty string is used to request a password prompt
        superdesk.Option("name"),
    ]

    def run(self, client_id, password, scope, name):
        self.validate_name(name)
        scope = self.validate_scope(scope)

        if client_id is None:
            client_id = ObjectId()
        else:
            client_id = self.validate_client_id(client_id)

        if not password or not password.strip():
            password = gen_password()

        client_data = {
            "_id": client_id,
            "name": name,
            "password": bcrypt.hashpw(password.encode(), bcrypt.gensalt()).decode(),
            "scope": list(scope),
        }

        superdesk.get_resource_service("auth_server_clients").post([client_data])
        print(
            "Client {name!r} has been registered with id '{client_id}' and password {password!r}".format(
                name=name, client_id=client_id, password=password
            )
        )

    def validate_client_id(self, client_id):
        """
        Validate client id string and return client id ObjectId
        :param client_id: client id
        :type client_id: str
        :return: client id
        :rtype: ObjectId
        """

        try:
            client_id = ObjectId(client_id)
        except InvalidId as e:
            self.parser.error("the given client id is not valid: {msg}".format(msg=e))

        try:
            next(superdesk.get_resource_service("auth_server_clients").find({"_id": client_id}))
        except StopIteration:
            pass
        else:
            self.parser.error("a client with this id already exists!")

        return client_id

    def validate_name(self, name):
        """
        Validate name
        :param name: client name
        :type name: str
        """

        if not name.strip():
            self.parser.error("please enter a valid name")

        try:
            next(superdesk.get_resource_service("auth_server_clients").find({"name": name}))
        except StopIteration:
            pass
        else:
            self.parser.error("a client with this name already exists!")


class UpdateClient(superdesk.Command, CommonClient):
    """Update an existing client

    You need to specify the client ID to update. Specify the parameters that you want to
    update. You can use ``--password`` to set a new password, keep empty to generate a new
    password.

    Examples:
    Change name of client with id ````:

        $ python manage.py auth_server:update_client 0102030405060708090a0b0c --name "some new name"

    Regenerate password of client with id ``5dad7ee94269dd1d5a78e6a1``:

        $ python manage.py auth_server:update_client 5dad7ee94269dd1d5a78e6a1 --password

    Change scope of client with id ``5dad7f064269dd1d5a78e6a2`` to allow only ARCHIVE_READ::

        $ python manage.py auth_server:update_client 5dad7f064269dd1d5a78e6a2 -s ARCHIVE_READ

    """

    option_list = [
        superdesk.Option("--name", "-n", help="change client name"),
        # empty string is used to re-generate password
        superdesk.Option(
            "--password", "-p", nargs="?", const="", help="update password (keep empty to re-generate it)"
        ),
        superdesk.Option(
            "--scope",
            "-s",
            action="append",
            default=[],
            help="scopes allowed (one or more of {allowed_scopes})".format(allowed_scopes=", ".join(allowed_scopes)),
        ),
        superdesk.Option("client_id"),
    ]

    def run(self, client_id, password, scope, name):
        clients_service = superdesk.get_resource_service("auth_server_clients")
        try:
            client_id = ObjectId(client_id)
        except InvalidId as e:
            self.parser.error("the given client id is not valid: {msg}".format(msg=e))

        try:
            original_client = next(clients_service.find({"_id": client_id}))
        except StopIteration:
            self.parser.error("Can't find any client with id '{client_id}'".format(client_id=client_id))

        client_updates = {}

        if name is not None and name.strip():
            client_updates["name"] = name

        if password is not None:
            if not password.strip():
                password = gen_password()
            client_updates["password"] = (bcrypt.hashpw(password.encode(), bcrypt.gensalt()).decode(),)

        if scope:
            client_updates["scope"] = self.validate_scope(scope)

        clients_service.update(original_client["_id"], client_updates.copy(), original_client)

        print("Client successfuly updated with:")
        for key, value in client_updates.items():
            if key == "scope":
                print("    {key}: {value}".format(key=key, value=", ".join(value)))
                continue
            elif key == "password":
                value = password

            print("    {key}: {value!r}".format(key=key, value=value))


class UnregisterClient(superdesk.Command):
    """Remove a previously registered client
    Example:

    Unregister client with id ``0102030405060708090a0b0c``::

        $ python manage.py auth_server:unregister_client 0102030405060708090a0b0c

    """

    option_list = [
        superdesk.Option("client_id"),
    ]

    def run(self, client_id):
        try:
            client_id = ObjectId(client_id)
        except InvalidId as e:
            self.parser.error("the given client id is not valid: {msg}".format(msg=e))
        clients_service = superdesk.get_resource_service("auth_server_clients")
        client = clients_service.find_one(req=None, _id=client_id)
        if client is None:
            print("No client with id '{client_id}' found".format(client_id=client_id))
            sys.exit(2)

        clients_service.delete({"_id": client_id})
        print("Client with id '{client_id}' has been successfuly unregistered".format(client_id=client_id))


class ListClients(superdesk.Command):
    """List clients registered with auth server

    The client will be listed with their scopes.

    Example::

        $ python manage.py auth_server:list_clients

    """

    def run(self):
        clients_service = superdesk.get_resource_service("auth_server_clients")
        clients_desc = []
        for client in clients_service.find({}):
            clients_desc.append(
                "- client {name!r}: id '{client_id}' with scope(s) {scopes}".format(
                    name=client["name"],
                    client_id=client["_id"],
                    scopes=", ".join(client["scope"]),
                )
            )

        if not clients_desc:
            print("No client currently registered with auth server")
        else:
            print("Following clients are currently registered:\n")
            print("\n".join(clients_desc))
            print()


superdesk.command("auth_server:register_client", RegisterClient())
superdesk.command("auth_server:update_client", UpdateClient())
superdesk.command("auth_server:unregister_client", UnregisterClient())
superdesk.command("auth_server:list_clients", ListClients())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
from superdesk.celery_app import celery
from .audit import AuditService, AuditResource
from .commands import PurgeAudit
import logging

log = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "audit"

    service = AuditService(endpoint_name, backend=superdesk.get_backend())
    AuditResource(endpoint_name, app=app, service=service)

    app.on_inserted += service.on_generic_inserted
    app.on_updated += service.on_generic_updated
    app.on_deleted_item += service.on_generic_deleted


@celery.task(soft_time_limit=600)
def gc_audit():
    PurgeAudit().run()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.utils import config
from copy import deepcopy

from superdesk import get_resource_service
from superdesk.resource import Resource
from superdesk.services import BaseService
from apps.archive.common import ITEM_UPDATE, get_user, ITEM_CREATE, ITEM_FETCH
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.utc import utcnow
from apps.item_lock.components.item_lock import LOCK_USER, LOCK_SESSION, LOCK_ACTION, LOCK_TIME

log = logging.getLogger(__name__)

fields_to_remove = [
    "_id",
    "_etag",
    "versioncreator",
    "originalcreator",
    "versioncreated",
    "_current_version",
    "version",
    "_updated",
    "lock_session",
    "lock_user",
    "lock_time",
    "lock_action",
    "force_unlock",
    "_created",
    "guid",
    "family_id",
    "firstcreated",
    "original_creator",
]


class ArchiveHistoryResource(Resource):
    endpoint_name = "archive_history"
    resource_methods = ["GET"]
    item_methods = ["GET"]
    schema = {
        "item_id": {"type": "string"},
        "user_id": Resource.rel("users", True),
        "operation": {"type": "string"},
        "update": {"type": "dict", "nullable": True},
        "version": {"type": "integer"},
        "original_item_id": {"type": "string"},
    }

    mongo_indexes = {
        "item_id": ([("item_id", 1)], {"background": True}),
        "_id_document": ([("_id_document", 1)], {"background": True}),
    }


class ArchiveHistoryService(BaseService):
    def on_item_updated(self, updates, original, operation=None):
        item = deepcopy(original)
        if updates:
            item.update(updates)

        if operation in [ITEM_CREATE, ITEM_FETCH]:
            # If this is the initial create on the item,
            # then store the metadata from original as well
            self._save_history(item, item, operation)
        else:
            self._save_history(item, updates, operation or ITEM_UPDATE)

    def on_item_deleted(self, doc):
        lookup = {"item_id": doc[config.ID_FIELD]}
        self.delete(lookup=lookup)

    def on_item_locked(self, item, user_id):
        self._save_lock_history(item, "item_lock")

    def on_item_unlocked(self, item, user_id):
        if item.get(config.VERSION, 1) == 0:
            # version 0 items get deleted on unlock, along with all history documents
            # so there is no need to record a history item here
            return

        self._save_lock_history(item, "item_unlock")

    def _save_lock_history(self, item, operation):
        self.post(
            [
                {
                    "item_id": item[config.ID_FIELD],
                    "user_id": self.get_user_id(item),
                    "operation": operation,
                    "update": {
                        LOCK_USER: item.get(LOCK_USER),
                        LOCK_SESSION: item.get(LOCK_SESSION),
                        LOCK_ACTION: item.get(LOCK_ACTION),
                        LOCK_TIME: item.get(LOCK_TIME),
                    },
                    "version": item.get(config.VERSION, 1),
                }
            ]
        )

    def get_user_id(self, item):
        user = get_user()
        if user:
            return user.get(config.ID_FIELD)

    def _save_history(self, item, update, operation):
        # in case of auto-routing, if the original_creator exists in our database
        # then create item create record in the archive history.
        if (
            item.get(ITEM_STATE) == CONTENT_STATE.ROUTED
            and item.get("original_creator")
            and not item.get("original_id")
        ):
            user = get_resource_service("users").find_one(req=None, _id=item.get("original_creator"))
            firstcreated = item.get("firstcreated", utcnow())

            if user:
                history = {
                    "item_id": item[config.ID_FIELD],
                    "user_id": user.get(config.ID_FIELD),
                    "operation": ITEM_CREATE,
                    "update": self._remove_unwanted_fields(update, item),
                    "version": item.get(config.VERSION, 1),
                    "_created": firstcreated,
                    "_updated": firstcreated,
                }

                self.post([history])

        history = {
            "item_id": item[config.ID_FIELD],
            "user_id": self.get_user_id(item),
            "operation": operation,
            "update": self._remove_unwanted_fields(update, item),
            "version": item.get(config.VERSION, 1),
        }

        self.post([history])

    def _remove_unwanted_fields(self, update, original):
        if update:
            update_copy = deepcopy(update)
            for field in fields_to_remove:
                update_copy.pop(field, None)

            if original.get("sms_message") == update_copy.get("sms_message"):
                update_copy.pop("sms_message", None)

            return update_copy



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from datetime import timedelta

from flask import current_app as app
from superdesk.celery_task_utils import get_lock_id
from superdesk.lock import lock, unlock
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class RemoveExportedFiles(superdesk.Command):
    """Remove files from storage that were used for exporting items

    Example:
    ::

        $ python manage.py storage:remove_exported
        $ python manage.py storage:remove_exported --expire-hours=12

    """

    log_msg = ""
    expire_hours = 24

    option_list = [superdesk.Option("--expire-hours", "-e", dest="expire_hours", required=False, type=int)]

    def run(self, expire_hours=None):
        if expire_hours:
            self.expire_hours = expire_hours
        elif "TEMP_FILE_EXPIRY_HOURS" in app.config:
            self.expire_hours = app.config["TEMP_FILE_EXPIRY_HOURS"]

        expire_at = utcnow() - timedelta(hours=self.expire_hours)
        self.log_msg = "Expiry Time: {}.".format(expire_at)
        logger.info("{} Starting to remove exported files from storage".format(self.log_msg))

        lock_name = get_lock_id("storage", "remove_exported")
        if not lock(lock_name, expire=300):
            logger.info("Remove exported files from storage task is already running")
            return

        try:
            logger.info("{} Removing expired temporary media files".format(self.log_msg))
            self._remove_exported_files(expire_at)
        finally:
            unlock(lock_name)

        logger.info("{} Completed removing exported files from storage".format(self.log_msg))

    def _remove_exported_files(self, expire_at):
        logger.info("{} Beginning to remove exported files from storage".format(self.log_msg))
        for file_id in self._get_file_ids(expire_at):
            app.media.delete(file_id)

    def _get_file_ids(self, expire_at):
        files = app.media.find(folder="temp", upload_date={"$lte": expire_at})
        return [file["_id"] for file in files]


superdesk.command("storage:remove_exported", RemoveExportedFiles())



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : superdesk
# Creation: 2017-08-14 11:47

from superdesk.commands.data_updates import BaseDataUpdate
from superdesk import get_resource_service
from eve.utils import config, app
from superdesk.factory.app import create_index
from superdesk.audit.commands import PurgeAudit


class DataUpdate(BaseDataUpdate):
    resource = "audit"

    def forwards(self, mongodb_collection, mongodb_database):
        for audit in mongodb_collection.find({"resource": {"$in": PurgeAudit.item_resources}}):
            audit_id = get_resource_service(self.resource)._extract_doc_id(audit.get("extra"))
            print(mongodb_collection.update({"_id": audit.get(config.ID_FIELD)}, {"$set": {"audit_id": audit_id}}))
        try:
            create_index(
                app=app,
                resource=self.resource,
                name="audit_id",
                list_of_keys=[("audit_id", 1)],
                index_options={"background": True},
            )
        except Exception:
            print("create index failed")

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update({}, {"$unset": {"audit_id": []}}, upsert=False, multi=True))



from flask import json
from superdesk.tests import TestCase, setup_auth_user


class PaginationTestCase(TestCase):
    headers = [("Content-Type", "application/json")]

    def test_default_pagination(self):
        setup_auth_user(self)
        self.app.data.insert("stages", [{"name": "stage{}".format(i)} for i in range(300)])

        response = self.client.get("/api/stages", headers=self.headers)
        self.assertEqual(200, response.status_code)

        data = json.loads(response.get_data())
        self.assertEqual(200, len(data["_items"]))
        self.assertEqual(300, data["_meta"]["total"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from flask import g
from superdesk.resource import Resource
from superdesk.services import BaseService

log = logging.getLogger(__name__)


class AuditResource(Resource):
    endpoint_name = "audit"
    resource_methods = ["GET"]
    item_methods = ["GET"]
    schema = {
        "resource": {"type": "string"},
        "action": {"type": "string"},
        "audit_id": {"type": "string"},
        "extra": {"type": "dict"},
        "user": Resource.rel("users", False),
    }
    exclude = {endpoint_name, "activity", "dictionaries", "macros", "archive_history", "formatters"}


class AuditService(BaseService):
    def on_generic_inserted(self, resource, docs):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, "user", None)
        if not user:
            if resource == "auth":
                user_id = docs[0].get("user")
            else:
                return
        else:
            user_id = user.get("_id")

        if not len(docs):
            return

        audit = {
            "user": user_id,
            "resource": resource,
            "action": "created",
            "extra": docs[0],
            "audit_id": self._extract_doc_id(docs[0]),
        }

        self.post([audit])

    def on_generic_updated(self, resource, doc, original):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, "user", None)
        if not user:
            return

        audit = {
            "user": user.get("_id"),
            "resource": resource,
            "action": "updated",
            "extra": doc,
            "audit_id": self._extract_doc_id(doc) if self._extract_doc_id(doc) else self._extract_doc_id(original),
        }
        if "_id" not in doc:
            audit["extra"]["_id"] = original.get("_id", None)
        self.post([audit])

    def on_generic_deleted(self, resource, doc):
        if resource in AuditResource.exclude:
            return

        user = getattr(g, "user", None)
        if not user:
            return

        audit = {
            "user": user.get("_id"),
            "resource": resource,
            "action": "deleted",
            "extra": doc,
            "audit_id": self._extract_doc_id(doc),
        }
        self.post([audit])

    def _extract_doc_id(self, doc):
        """
        Given an audit item try to extract the id of the item that it relates to
        :param item:
        :return:
        """
        try:
            id = doc.get("_id", doc.get("guid", doc.get("item_id", doc.get("item", None))))
            # do not return an id for items that have a dictionary id
            if not isinstance(id, dict):
                return id
            else:
                None
        except Exception:
            return None
        return None



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from nose.tools import assert_raises

from superdesk.tests import TestCase
from superdesk.macros.validate_for_publish import validate_for_publish, ValidationError


class ValidateForPublishTests(TestCase):
    validator = {
        "_id": "publish_text",
        "act": "publish",
        "type": "text",
        "schema": {"headline": {"required": True, "maxlength": 5, "empty": False, "nullable": False, "type": "string"}},
    }

    def test_validator(self):
        self.app.data.insert("archive", [{"_id": 1, "type": "text", "headline": "123456"}])
        self.app.data.insert("validators", [self.validator])
        with self.app.app_context():
            with assert_raises(ValidationError):
                item = {"_id": 1}
                validate_for_publish(item)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import pytz
from superdesk.etree import etree
import html
from superdesk.io.feed_parsers.newsml_1_2 import NewsMLOneFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.errors import ParserError
from dateutil.parser import parse as date_parser
from flask import current_app as app
from apps.archive.common import format_dateline_to_locmmmddsrc


class ANANewsMLOneFeedParser(NewsMLOneFeedParser):
    """ANA specific NewsML parser.

    Feed Parser which can parse the ANA English feed basically it is in NewsML 1.2 format
    """

    NAME = "ananewsml12"

    label = "ANA NewsML Parser"

    def can_parse(self, xml):
        return xml.tag == "NewsML"

    def parse(self, xml, provider=None):
        item = {}
        try:
            self.root = xml
            parsed_el = xml.find("NewsItem/NewsComponent/AdministrativeMetadata/Source/Party")
            if parsed_el is not None:
                item["original_source"] = parsed_el.attrib.get("FormalName", "ANA")

            parsed_el = xml.find("NewsEnvelope/Priority")
            item["priority"] = self.map_priority(parsed_el.text if parsed_el is not None else None)

            self.parse_news_identifier(item, xml)
            self.parse_newslines(item, xml)
            self.parse_news_management(item, xml)

            parsed_el = xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/Language")
            if parsed_el is not None:
                language = self.parse_attributes_as_dictionary(parsed_el)
                item["language"] = language[0]["FormalName"] if len(language) else ""

            subjects = xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectDetail[@Scheme="IptcSubjectCodes"]'
            )
            subjects += xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectMatter[@Scheme="IptcSubjectCodes"]'
            )
            subjects += xml.findall(
                'NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/Subject[@Scheme="IptcSubjectCodes"]'
            )

            item["subject"] = self.format_subjects(subjects)

            item["body_html"] = (
                html.unescape(
                    etree.tostring(
                        xml.find("NewsItem/NewsComponent/NewsComponent/ContentItem/DataContent"), encoding="unicode"
                    )
                )
                .replace("<DataContent>", "")
                .replace("</DataContent>", "")
                .replace("<P>", "<p>")
                .replace("</P>", "</p>")
            )

            item["body_html"] = (
                item.get("body_html")
                .replace(
                    "<p>© ΑΠΕ-ΜΠΕ ΑΕ. Τα πνευματικά δικαιώματα ανήκουν στο "
                    "ΑΠΕ-ΜΠΕ ΑΕ και παραχωρούνται σε συνδρομητές μόνον "
                    "για συγκεκριμένη χρήση.</p>",
                    "",
                )
                .strip()
            )
            parsed_el = xml.findall("NewsItem/NewsComponent/NewsComponent/ContentItem/Characteristics/Property")
            characteristics = self.parse_attribute_values(parsed_el, "WordCount")
            item["word_count"] = characteristics[0] if len(characteristics) else None

            # Extract the city for setting into the dateline
            city = xml.find('NewsItem/NewsComponent/DescriptiveMetadata/Property[@FormalName="City"]').attrib.get(
                "Value"
            )
            # Anglicise the greek for Athens if required
            city = "Athens" if city == "Αθήνα" else city
            country = xml.find('NewsItem/NewsComponent/DescriptiveMetadata/Property[@FormalName="Country"]').attrib.get(
                "Value"
            )
            # Normalise the country code
            country = "GR" if country == "GRC" else country

            cities = app.locators.find_cities()
            located = [c for c in cities if c["city"] == city and c["country_code"] == country]
            if len(located) == 1:
                item["dateline"]["located"] = located[0]
                item["dateline"]["source"] = provider.get("source")
                item["dateline"]["text"] = format_dateline_to_locmmmddsrc(
                    item["dateline"]["located"], item.get("dateline", {}).get("date"), provider.get("source")
                )
            return self.populate_fields(item)
        except Exception as ex:
            raise ParserError.newsmlOneParserError(ex, provider)

    def parse_newslines(self, item, tree):
        parsed_el = self.parse_elements(tree.find("NewsItem/NewsComponent/NewsLines"))

        # Set the date component of the dateline assuming that the date is in the Athens timezone
        local_dt = date_parser(parsed_el.get("DateLine", ""), fuzzy=True, dayfirst=True)
        local_tz = pytz.timezone("Europe/Athens")
        aus_dt = local_tz.localize(local_dt, is_dst=None)
        item.setdefault("dateline", {})
        item["dateline"]["date"] = aus_dt.astimezone(pytz.utc)

        item["headline"] = parsed_el.get("HeadLine", "").strip()
        item["slugline"] = parsed_el.get("SlugLine", "")
        # The byline is not extracted as it seems to use the Greek character set

        return True

    def parse_news_management(self, item, tree):
        parsed_el = self.parse_elements(tree.find("NewsItem/NewsManagement"))
        item["urgency"] = int(parsed_el.get("Urgency", {}).get("FormalName", 3))
        item["versioncreated"] = self.datetime(parsed_el["ThisRevisionCreated"])
        item["firstcreated"] = self.datetime(parsed_el["FirstCreated"])
        item["pubstatus"] = (parsed_el["Status"]["FormalName"]).lower()

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, "%Y%m%dT%H%M%S%z")
        except ValueError:
            return super().datetime(string)


register_feed_parser(ANANewsMLOneFeedParser.NAME, ANANewsMLOneFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.metadata.utils import item_url


class SessionsResource(Resource):
    schema = {
        "user": Resource.rel("users", True),
        "session_preferences": {"type": "dict"},
    }
    datasource = {"source": "auth", "default_sort": [("_created", -1)]}
    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "DELETE", "PATCH"]
    embedded_fields = ["user"]


class UserSessionClearResource(Resource):
    endpoint_name = "clear_sessions"
    url = "users/<{0}:user>/sessions".format(item_url)
    datasource = {"source": "auth"}
    resource_methods = ["DELETE"]
    item_methods = []
    resource_title = endpoint_name
    privileges = {"DELETE": "users"}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from . import currency_base
from decimal import Decimal
from flask_babel import lazy_gettext

USD_TO_CAD = Decimal("1.3139")  # backup


def get_rate():
    """Get USD to CAD rate."""
    try:
        return currency_base.get_rate("USD", "CAD")
    except Exception:
        return USD_TO_CAD


def usd_to_cad(item, **kwargs):
    """Convert USD to CAD."""

    rate = kwargs.get("rate") or get_rate()
    if os.environ.get("BEHAVE_TESTING"):
        rate = Decimal(2)

    return currency_base.do_conversion(item, rate, "CAD ", "\$([0-9]+)", match_index=0, value_index=1)


name = "usd_to_cad"
label = lazy_gettext("Currency USD to CAD")
shortcut = "d"
callback = usd_to_cad
access_type = "frontend"
action_type = "interactive"
from_languages = ["en-CA"]
to_languages = ["en-AU"]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .service import ItemsVersionsService
from .resource import ItemsVersionsResource


def init_app(app):
    """Initialise the items versions end point

    :param app: the API application object
    :type app: `Eve`
    """
    endpoint_name = "items_versions"
    service = ItemsVersionsService(endpoint_name, backend=superdesk.get_backend())
    ItemsVersionsResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : superdesk
# Creation: 2017-02-14 14:53

from superdesk.commands.data_updates import BaseDataUpdate
from superdesk import get_resource_service


class DataUpdate(BaseDataUpdate):

    resource = "vocabularies"
    product_types = {
        "_id": "product_types",
        "display_name": "Product Types",
        "type": "unmanageable",
        "items": [
            {"is_active": True, "name": "API", "qcode": "api"},
            {"is_active": True, "name": "Direct", "qcode": "direct"},
            {"is_active": True, "name": "Both", "qcode": "both"},
        ],
    }

    def forwards(self, mongodb_collection, mongodb_database):
        product_types = get_resource_service(self.resource).find_one(req=None, _id="product_types")
        if product_types:
            print("Product Types vocabulary already exists in the system.")
            return
        get_resource_service(self.resource).post([self.product_types])

    def backwards(self, mongodb_collection, mongodb_database):
        get_resource_service(self.resource).delete_action({"_id": "product_types"})



import os
from unittest import mock

from superdesk import get_resource_service
from superdesk.io.feeding_services import FTPFeedingService
from superdesk.io.commands.update_ingest import ingest_items

from .mocks_reuters import setup_reuters_mock, teardown_reuters_mock


def setup_providers(context):
    app = context.app
    context.providers = {}
    context.ingest_items = ingest_items
    with app.test_request_context(app.config["URL_PREFIX"]):
        rule_sets = {
            "name": "reuters_rule_sets",
            "rules": [
                {"old": "@", "new": ""},
            ],
        }

        result = get_resource_service("rule_sets").post([rule_sets])

        app.config["REUTERS_USERNAME"] = "no_username"
        app.config["REUTERS_PASSWORD"] = "no_password"
        setup_reuters_mock(context)

        path_to_fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), "../../tests/io/fixtures")
        providers = [
            {
                "name": "reuters",
                "source": "reuters",
                "feeding_service": "reuters_http",
                "feed_parser": "newsml2",
                "is_closed": False,
                "rule_set": result[0],
                "config": {
                    "url": "http://rmb.reuters.com/rmd/rest/xml",
                    "auth_url": "https://commerce.reuters.com/rmd/rest/xml/login",
                    "username": app.config["REUTERS_USERNAME"],
                    "password": app.config["REUTERS_PASSWORD"],
                },
            },
            {
                "name": "AAP",
                "source": "AAP Ingest",
                "feeding_service": "file",
                "feed_parser": "nitf",
                "is_closed": False,
                "critical_errors": {"2005": True},
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "teletype",
                "source": "AAP Teletype",
                "feeding_service": "file",
                "feed_parser": "zczc",
                "is_closed": False,
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "DPA",
                "source": "DPA",
                "feeding_service": "file",
                "feed_parser": "dpa_iptc7901",
                "is_closed": False,
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "AP",
                "source": "AP",
                "feeding_service": "file",
                "feed_parser": "ap_anpa1312",
                "is_closed": False,
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "STT",
                "source": "STT",
                "feeding_service": "file",
                "feed_parser": "sttnewsml",
                "is_closed": False,
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "ninjs",
                "source": "NINJS",
                "feeding_service": "file",
                "feed_parser": "ninjs",
                "is_closed": False,
                "config": {"path": path_to_fixtures},
            },
            {
                "name": "email",
                "source": "Email",
                "feeding_service": "file",
                "feed_parser": "email_rfc822",
                "is_closed": False,
                "config": {"path": path_to_fixtures, "formatted": True},
            },
            {
                "name": "ftp_ninjs",
                "source": "ftp ninjs",
                "feeding_service": "ftp",
                "feed_parser": "ninjs",
                "is_closed": False,
                "config": {
                    "path_fixtures": path_to_fixtures,
                    "formatted": True,
                    "host": "fake-ftp.example.com",
                    "username": "admin",
                    "password": "admin",
                    "path": "/admin/superdesk",
                    "dest_path": "/tmp",
                },
            },
        ]

        with mock.patch.object(FTPFeedingService, "_test", return_value=True):
            result = get_resource_service("ingest_providers").post(providers)

        context.providers["reuters"] = result[0]
        context.providers["aap"] = result[1]
        context.providers["teletype"] = result[2]
        context.providers["dpa"] = result[3]
        context.providers["ap"] = result[4]
        context.providers["ninjs"] = result[5]
        context.providers["email"] = result[6]
        context.providers["ftp_ninjs"] = result[7]


def teardown_providers(context):
    teardown_reuters_mock(context)



import io
import flask
import unittest
import requests_mock

from superdesk.media.media_operations import download_file_from_url


class MediaOperationsTestCase(unittest.TestCase):
    def test_download_file_from_url_relative(self):
        app = flask.Flask(__name__)
        app.config["SERVER_NAME"] = "localhost"
        body = io.BytesIO(b"data")
        with app.app_context():
            with requests_mock.mock() as mock:
                mock.get("http://localhost/test/foo.jpg", body=body)
                out = download_file_from_url("/test/foo.jpg")
        self.assertEqual(b"data", out[0].read())



"""
Superdesk Google Authentication

.. versionadded:: 1.8

You can use Google for authentication, first you have to create credentials
in `Google API console <https://console.developers.google.com/apis/credentials>`_:

- set your client URL as *Authorized JavaScript origins*::

    https://example.com

- set server URL + ``/api/login/google_authorized`` as *Authorized redirect URIs*::

    https://example.com/api/login/google_authorized

Once configured you will find there *Client ID* and *Client secret*, use both to populate :ref:`settings.google_oauth`.

.. versionchanged:: 1.9
    There is no need to configure client, it reads config from server now.

.. versionchanged:: 1.9
    Login url is ``/api/login/google_authorized`` instead of ``/login/google_authorized``.

"""

import logging
from datetime import datetime
from typing import Optional, List
import superdesk
from flask import url_for, render_template
from authlib.integrations.flask_client import OAuth
from authlib.integrations.requests_client import OAuth2Session
from authlib.oauth2.rfc6749.wrappers import OAuth2Token
from superdesk.resource import Resource
from superdesk.services import BaseService

from superdesk.auth import auth_user, TEMPLATE


logger = logging.getLogger(__name__)

bp = superdesk.Blueprint("oauth", __name__)
oauth: Optional[OAuth] = None
REFRESH_TOKEN_URL = "https://oauth2.googleapis.com/token"


def init_app(app):
    global oauth
    oauth = OAuth(app)

    endpoint_name = "oauth2_token"
    service = OAuth2TokenService(endpoint_name, backend=superdesk.get_backend())
    OAuth2TokenResource(endpoint_name, app=app, service=service)

    app.client_config["google_auth"] = False
    if app.config.get("GOOGLE_CLIENT_ID") and app.config.get("GOOGLE_CLIENT_SECRET"):
        app.client_config["google_auth"] = bool(app.config.get("GOOGLE_LOGIN"))
        if app.config.get("GOOGLE_GMAIL"):
            extra_scopes = ["https://mail.google.com"]
            refresh = True
        else:
            extra_scopes = []
            refresh = False

        configure_google(app, extra_scopes=extra_scopes, refresh=refresh)


class OAuth2TokenResource(Resource):
    schema = {
        "name": {
            "type": "string",
            "required": True,
            "nullable": False,
            "empty": False,
        },
        "email": {
            "type": "string",
        },
        "access_token": {
            "type": "string",
            "required": True,
            "nullable": False,
            "empty": False,
        },
        "refresh_token": {
            "type": "string",
        },
        "expires_at": {"type": "datetime"},
    }
    internal_resource = True


class OAuth2TokenService(BaseService):
    """Store OAuth tokens"""


def token2dict(
    token_id: str,
    email: str,
    token: OAuth2Token,
    name: str = "google",
) -> dict:
    """Convert authlib's OAuth2Token to a dict usable with datalayer

    :param token_id: used to associate the token with a provider
    :param email: email associated with this token
    :param token: token to convert
    :param name: name of the OAuth2 service
    """
    return {
        "_id": token_id,
        "name": "google",
        "email": email,
        "access_token": token["access_token"],
        "refresh_token": token.get("refresh_token"),
        "expires_at": datetime.fromtimestamp(token["expires_at"]),
    }


def configure_google(app, extra_scopes: Optional[List[str]] = None, refresh: bool = False) -> None:
    scopes = ["openid", "email", "profile"]
    token_url_id_queue = []
    if extra_scopes:
        scopes.extend(extra_scopes)
    kwargs = {}
    if refresh:
        kwargs["authorize_params"] = {"access_type": "offline"}

    oauth.register(  # type: ignore # mypy seems confused with this global oauth
        "google",
        server_metadata_url="https://accounts.google.com/.well-known/openid-configuration",
        client_kwargs={"scope": " ".join(scopes)},
        **kwargs,
    )

    @bp.route("/login/google")
    @bp.route("/login/google/<url_id>")
    def google_login(url_id=None):
        """Redirect to google OAuth authorization page

        :param url_id: used to identify the token
            if OAuth is used for Superdesk login, url_id is None.
            Otherwise, it is used to associate the token with the provider needing it
        """
        token_url_id_queue.append(url_id)
        redirect_uri = url_for(".google_authorized", _external=True)
        return oauth.google.authorize_redirect(redirect_uri)

    @bp.route("/login/google_authorized")
    def google_authorized():
        token_id = token_url_id_queue.pop() if token_url_id_queue else None
        token = oauth.google.authorize_access_token()
        if not token:
            return render_template(TEMPLATE, data={}) if token_id else auth_user()
        user = oauth.google.parse_id_token(token)
        if token_id:
            # token_id is used to link token with provider, we need to store the token
            # in this case, to be able to use it and refresh later without user interaction
            oauth2_token_service = superdesk.get_resource_service("oauth2_token")
            if token.get("refresh_token"):
                oauth2_token_service.post([token2dict(token_id, user["email"], token)])
            else:
                # we have no refresh_token, that probably means that's it not the first time that we log in
                # with OAuth (refresh token is only returned on first token exchange). We should already have
                # a token in database, let's check it.
                current_token = oauth2_token_service.find_one(req=None, _id=token_id)
                if current_token:
                    if current_token["access_token"] != token["access_token"]:
                        # we have a new access_token, we update it, but we want to keep existing refresh_token
                        token_dict = token2dict(token_id, user["email"], token)
                        oauth2_token_service.update(
                            token_id,
                            {"access_token": token_dict["access_token"], "expires_at": token_dict["expires_at"]},
                            current_token,
                        )
                else:
                    logger.warning(
                        "No refresh token received, that probably means that it's not the first time login is "
                        "requested. Please remove granted permission to Superdesk in Google settings (under "
                        '"security/Third-party apps with account access") then try to log-in again'
                    )

            return render_template(TEMPLATE, data={})
        else:
            # no token_id, OAuth is only used for log-in
            return auth_user(user["email"], {"needs_activation": False})

    superdesk.blueprint(bp, app)


def refresh_google_token(token_id: str) -> dict:
    oauth2_token_service = superdesk.get_resource_service("oauth2_token")
    token = oauth2_token_service.find_one(req=None, _id=token_id)
    if token is None:
        raise ValueError("unknown token id: {_id}".format(_id=token_id))
    if not token["refresh_token"]:
        raise ValueError("missing refresh token for token {_id}".format(_id=token["_id"]))
    session = OAuth2Session(
        oauth.google.client_id,  # type: ignore # mypy seems confused with this global oauth
        oauth.google.client_secret,  # type: ignore
    )
    new_token = session.refresh_token(REFRESH_TOKEN_URL, token["refresh_token"])
    token_dict = token2dict(token["_id"], token["email"], new_token)
    oauth2_token_service.update(token["_id"], token_dict, token)
    return token_dict



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re

from decimal import Decimal
from superdesk.macros import macro_replacement_fields


def format_converted(converted_value, precision):
    if converted_value > Decimal(1000):
        precision = 0

    if converted_value < Decimal(1) and precision == 0:
        precision = 2

    rounded = round(converted_value, precision)
    if rounded == Decimal(0):
        precision += 1
        rounded = round(converted_value, precision)

    return "{0:,}".format(round(converted_value, precision))


def format_output(original, converted, symbol):
    """Returns the replacement string for the given original value"""
    return "{} ({} {})".format(original, converted, symbol)


def do_conversion(item, converter, formatter, search_param, match_index, value_index):
    """Performs the conversion
    :param item: story
    :param converter: function to perform conversion
    :param formatter: function to do string formatting
    :param search_param: search parameter to locate the original value.  It should
    be a valid regular expression pattern, and not just an arbitrary string.
    :param match_index: int index of groups used in matching string
    :param value_index: int index of groups used in converting the value
    :return: modified story
    """
    diff = {}

    def convert(match):
        match_item = match.group(match_index)
        from_value = match.group(value_index)
        multi_values = "-" in from_value and from_value[-1:] != "-"
        precision = 0

        if match_item and from_value:
            if not multi_values:
                from_value = re.sub(r"[^\d.]", "", from_value)
                precision = abs(Decimal(from_value).as_tuple().exponent)
            to_value, symbol = converter(from_value, precision=precision)
            diff.setdefault(match_item, formatter(match_item, to_value, symbol))
            return diff[match_item]

    for field in macro_replacement_fields:
        if item.get(field, None):
            re.sub(search_param, convert, item[field])

    return (item, diff)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .volume_cubic_feet_to_metric import cubic_feet_to_metric


class VolumeTestCase(unittest.TestCase):
    def test(self):
        text = (
            "Total volume is 100.50 cubic feet for this land. "
            "Total volume is 15.7 cubic ft for this land. "
            "Total volume is 1 Cubic Foot for this land. "
            "Total volume is 1-16 cu-ft for this land. "
            "Total volume is 1-16 cb. ft for this land. "
            "Total volume is 16.7-Cubic-ft for this land. "
            "Total volume is 16,500-cu. ft. for this land. "
        )
        item = {"body_html": text}
        res, diff = cubic_feet_to_metric(item)
        self.assertEqual(diff["100.50 cubic feet"], "100.50 cubic feet (2.85 cubic meter)")
        self.assertEqual(diff["15.7 cubic ft"], "15.7 cubic ft (0.4 cubic meter)")
        self.assertEqual(diff["1 Cubic Foot"], "1 Cubic Foot (0.03 cubic meter)")
        self.assertEqual(diff["1-16 cu-ft"], "1-16 cu-ft (0.03-0.45 cubic meter)")
        self.assertEqual(diff["1-16 cb. ft"], "1-16 cb. ft (0.03-0.45 cubic meter)")
        self.assertEqual(diff["16.7-Cubic-ft"], "16.7-Cubic-ft (0.5 cubic meter)")
        self.assertEqual(diff["16,500-cu. ft"], "16,500-cu. ft (467 cubic meter)")
        self.assertEqual(res["body_html"], item["body_html"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from nose.tools import assert_raises

from apps.archive.common import get_item_expiry
from apps.tasks import (
    apply_stage_rule,
    apply_onstage_rule,
    compare_dictionaries,
    MACRO_INCOMING,
    MACRO_OUTGOING,
    MACRO_ONSTAGE,
)
from superdesk.errors import SuperdeskApiError
from superdesk.utc import get_expiry_date
from superdesk.tests import TestCase


class TasksTestCase(TestCase):
    def test_get_global_content_expiry(self):
        calculated_minutes = get_item_expiry(desk=None, stage=None)
        reference_minutes = get_expiry_date(self.ctx.app.config["CONTENT_EXPIRY_MINUTES"])
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_get_desk_content_expiry(self):
        desk = {"content_expiry": 10}
        calculated_minutes = get_item_expiry(desk=desk, stage=None)
        reference_minutes = get_expiry_date(10)
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_get_stage_content_expiry(self):
        stage = {"content_expiry": 10}
        calculated_minutes = get_item_expiry(desk=None, stage=stage)
        reference_minutes = get_expiry_date(10)
        self.assertEquals(calculated_minutes.hour, reference_minutes.hour)
        self.assertEquals(calculated_minutes.minute, reference_minutes.minute)

    def test_apply_incoming_stage_rule(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"anpa_take_key": "x"}
        stage = {"incoming_macro": "populate_abstract"}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)
            self.assertEquals(update["abstract"], "Test-1")

    def test_apply_outgoing_stage_rule(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"anpa_take_key": "x"}
        stage = {"outgoing_macro": "populate_abstract"}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_OUTGOING)
            self.assertEquals(update["abstract"], "Test-1")

    def test_apply_stage_incoming_validation_rule(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"headline": "x"}
        stage = {"incoming_macro": "take_key_validator"}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_incoming_validation_rule_passes(self):
        doc = {"id": "1", "body_html": "Test-1", "anpa_take_key": "a"}
        update = {"headline": "x"}
        stage = {"incoming_macro": "take_key_validator"}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_incoming_validation_rule_ignored(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"headline": "x"}
        stage = {"outgoing_macro": "take_key_validator"}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_INCOMING)

    def test_apply_stage_outgoing_validation_rule_ignored(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"headline": "x"}
        stage = {"incoming_macro": "take_key_validator"}
        with self.app.app_context():
            apply_stage_rule(doc, update, stage, MACRO_OUTGOING)

    def test_apply_stage_outgoing_validation_rule(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"headline": "x"}
        stage = {"outgoing_macro": "take_key_validator"}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_OUTGOING)

    def test_apply_on_stage_validation_rule(self):
        doc = {"id": "1", "body_html": "Test-1"}
        update = {"headline": "x"}
        stage = {"onstage_macro": "take_key_validator"}
        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_stage_rule(doc, update, stage, MACRO_ONSTAGE)

    def test_apply_onstage_rule(self):
        doc = {"id": "1", "body_html": "Test-1", "task": {"stage": 1}}
        stages = [{"_id": 1, "onstage_macro": "take_key_validator"}]
        self.app.data.insert("stages", stages)

        with self.app.app_context():
            with assert_raises(SuperdeskApiError):
                apply_onstage_rule(doc, 1)

    def test_apply_onstage_rule_applies(self):
        doc = {"id": "1", "body_html": "Test-1", "task": {"stage": 1}}
        stages = [{"_id": 1, "onstage_macro": "populate_abstract"}]
        self.app.data.insert("stages", stages)

        with self.app.app_context():
            apply_onstage_rule(doc, 1)
            self.assertEquals(doc["abstract"], "Test-1")

    def test_compare_dictionaries(self):
        original = {"id": 1, "body_html": "Test-1"}

        updates = {
            "body_html": "Test-2",
            "headline": "a",
        }

        modified = compare_dictionaries(original, updates)
        self.assertEquals(2, len(modified))
        self.assertTrue("body_html" in modified)
        self.assertTrue("headline" in modified)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
import unittest

from unittest.mock import patch
from datetime import datetime, timedelta

from apps.templates.filters import format_datetime_filter
from apps.templates.content_templates import get_item_from_template, render_content_template


class RenderTemplateTestCase(unittest.TestCase):
    def setUp(self):
        self.app = flask.Flask(__name__)
        self.app.app_context().push()
        self.app.jinja_env.filters["format_datetime"] = format_datetime_filter

    def test_render_content_template(self):
        template = {
            "_id": "foo",
            "template_name": "test",
            "template_desks": ["sports"],
            "data": {
                "headline": "Foo Template: {{item.headline}}",
                "body_html": 'This article has slugline: {{item.slugline}} and dateline: {{item.dateline["text"]}} '
                'at {{item.versioncreated | format_datetime("Australia/Sydney", "%d %b %Y %H:%S %Z")}}',
                "urgency": 1,
                "priority": 3,
                "dateline": {},
                "anpa_take_key": "this is test",
                "place": ["Australia"],
            },
        }

        item = {
            "_id": "123",
            "headline": "Test Template",
            "slugline": "Testing",
            "body_html": "This is test story",
            "dateline": {"text": "hello world"},
            "urgency": 4,
            "priority": 6,
            "versioncreated": "2015-06-01T22:54:53+0000",
            "place": ["NSW"],
        }

        updates = render_content_template(item, template)
        self.assertEqual(updates["headline"], "Foo Template: Test Template")
        self.assertEqual(updates["urgency"], 1)
        self.assertEqual(updates["priority"], 3)
        self.assertEqual(
            updates["body_html"],
            "This article has slugline: Testing and dateline: " "hello world at 02 Jun 2015 08:53 AEST",
        )
        self.assertListEqual(updates["place"], ["Australia"])

    def test_headline_strip_tags(self):
        template = {"data": {"headline": " test\nit<br>"}}

        updates = render_content_template({}, template)
        self.assertEqual("test it", updates["headline"])

        item = get_item_from_template(template)
        self.assertEqual("test it", item["headline"])

    def test_render_dateline_current_time(self):
        now = datetime(2020, 12, 8, 13, 0, 0)
        template = {
            "data": {
                "dateline": {
                    "located": {
                        "dateline": "city",
                        "tz": "Europe/Prague",
                        "city": "Prague",
                        "city_code": "Prague",
                        "country_code": "CZ",
                        "state_code": "52",
                    },
                    "date": now - timedelta(days=5),
                    "text": "PRAGUE, Dec 3 -",
                },
            }
        }

        with patch("apps.templates.content_templates.utcnow", return_value=now):
            updates = render_content_template({}, template)
        self.assertEqual("PRAGUE, Dec 8  -", updates["dateline"]["text"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import datetime
from unittest import mock
from lxml import etree

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from superdesk.publish import init_app
from superdesk.publish.formatters.newsml_1_2_formatter import NewsML12Formatter


@mock.patch("superdesk.publish.subscribers.SubscribersService.generate_sequence_number", lambda self, subscriber: 1)
class Newsml12FormatterTest(TestCase):
    article = {
        "_id": "urn:localhost.abc",
        "guid": "urn:localhost.abc",
        "source": "AAP",
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
        "headline": "This is a test headline - для теста!",
        "byline": "joe",
        "slugline": "slugline",
        "subject": [{"qcode": "02011001"}, {"qcode": "02011002"}],
        "anpa_take_key": "take_key",
        "unique_id": "1",
        "body_html": "<p>The story body&nbsp;more to the story - больше историй</p>",
        "type": "text",
        "word_count": "1",
        "priority": 1,
        "_current_version": 5,
        "state": "published",
        "urgency": 2,
        "pubstatus": "usable",
        "dateline": {
            "source": "AAP",
            "text": "sample dateline",
            "located": {
                "alt_name": "",
                "state": "California",
                "city_code": "Los Angeles",
                "city": "Los Angeles",
                "dateline": "city",
                "country_code": "US",
                "country": "USA",
                "tz": "America/Los_Angeles",
                "state_code": "CA",
            },
        },
        "keywords": ["traffic"],
        "abstract": "sample abstract",
        "place": [
            {
                "qcode": "NSW",
                "name": "NSW",
                "state": "New South Wales",
                "country": "Australia",
                "world_region": "Oceania",
            }
        ],
        "ednote": "this is test",
        "body_footer": "<p>call helpline 999 if you are planning to quit smoking</p>",
        "company_codes": [{"name": "YANCOAL AUSTRALIA LIMITED", "qcode": "YAL", "security_exchange": "ASX"}],
    }

    preformatted = {
        "_id": "urn:localhost.123",
        "guid": "urn:localhost.123",
        "source": "AAP",
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
        "headline": "This is a test headline - для теста!",
        "byline": "joe",
        "slugline": "slugline",
        "subject": [{"qcode": "02011001"}, {"qcode": "02011002"}],
        "anpa_take_key": "take_key",
        "unique_id": "1",
        "type": "preformatted",
        "body_html": "The story body",
        "word_count": "1",
        "_current_version": 5,
        "state": "published",
        "urgency": 2,
        "pubstatus": "usable",
        "dateline": {"text": "sample dateline"},
        "keywords": ["traffic"],
        "abstract": "sample abstract",
        "place": [
            {"qcode": "Australia", "name": "Australia", "state": "", "country": "Australia", "world_region": "Oceania"}
        ],
    }

    picture = {
        "_id": "tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3",
        "guid": "tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3",
        "firstcreated": "2015-09-20T06:12:57.000Z",
        "versioncreated": "2015-09-20T06:14:11.000Z",
        "dateline": {"source": "AAP", "date": "2015-08-14T04:45:53.000Z"},
        "renditions": {
            "viewImage": {
                "height": 415,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773786/raw?_schema=http",
                "media": "55cd72811d41c828e1773786",
                "mimetype": "image/jpeg",
                "width": 640,
            },
            "baseImage": {
                "height": 909,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773782/raw?_schema=http",
                "media": "55cd72811d41c828e1773782",
                "mimetype": "image/jpeg",
                "width": 1400,
            },
            "thumbnail": {
                "height": 120,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773784/raw?_schema=http",
                "media": "55cd72811d41c828e1773784",
                "mimetype": "image/jpeg",
                "width": 184,
            },
            "original": {
                "height": 2455,
                "href": "http://localhost:5000/api/upload/55cd72801d41c828e1773762/raw?_schema=http",
                "media": "55cd72801d41c828e1773762",
                "mimetype": "image/jpeg",
                "width": 3777,
            },
        },
        "state": "published",
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
        "source": "AAP Image",
        "_current_version": 1,
        "original_source": "AAP Image/AAP",
        "description": "Federal Education Minister Christopher Pyne launches his new book NO ARCHIVING",
        "type": "picture",
        "slugline": "NUS CHRISTOPHER PYNE PROTEST",
        "headline": "NUS CHRISTOPHER PYNE PROTEST",
        "pubstatus": "usable",
        "ednote": "",
        "byline": "TRACEY NEARMY",
        "filemeta": {
            "yresolution": [300, 1],
            "exposuretime": [1, 200],
            "copyright": "                                                      ",
            "scenecapturetype": 0,
            "sensingmethod": 2,
            "fnumber": [14, 5],
            "flashpixversion": "0100",
            "xresolution": [300, 1],
            "resolutionunit": 2,
            "subsectimedigitized": "20",
            "exposureprogram": 1,
            "subsectimeoriginal": "20",
            "make": "NIKON CORPORATION",
            "focallengthin35mmfilm": 200,
            "scenetype": 1,
            "exifimageheight": 2455,
            "saturation": 0,
            "colorspace": 1,
            "subjectdistancerange": 0,
            "datetime": "2015:07:31 18:55:37",
            "software": "Photogene for iPad v4.3",
            "flash": 16,
            "focallength": [200, 1],
            "componentsconfiguration": "\u0001\u0002\u0003\u0000",
            "lightsource": 3,
            "artist": "                                    ",
            "isospeedratings": 2000,
            "whitepoint": [313, 1000],
            "sharpness": 2,
            "exposuremode": 1,
            "meteringmode": 3,
            "compressedbitsperpixel": [4, 1],
            "model": "NIKON D800E",
            "subsectime": "20",
            "datetimedigitized": "2015:07:31 18:55:37",
            "exifoffset": 406,
            "contrast": 0,
            "whitebalance": 1,
            "exifimagewidth": 3777,
            "datetimeoriginal": "2015:07:31 18:55:37",
            "customrendered": 0,
            "maxaperturevalue": [3, 1],
            "digitalzoomratio": [1, 1],
            "primarychromaticities": [16, 25],
            "length": 8009209,
            "exifversion": "0230",
            "gaincontrol": 2,
            "gamma": [11, 5],
            "filesource": 3,
        },
        "language": "en",
        "mimetype": "image/jpeg",
        "sign_off": "mar",
        "unique_id": 573,
    }

    video = {
        "_id": "urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e",
        "guid": "tag:localhost:2015:c11e11c4-cdbc-41ef-b939-2b30dd8365fb",
        "language": "en",
        "family_id": "urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e",
        "_current_version": 3,
        "versioncreated": "2015-09-20T06:14:11.000Z",
        "unique_id": 274,
        "renditions": {
            "original": {
                "media": "55fe4e691d41c8cac923ceb2",
                "href": "http://192.168.220.176:5000/api/upload/55fe4e691d41c8cac923ceb2/raw?_schema=http",
                "mimetype": "video/mp4",
            }
        },
        "state": "in_progress",
        "version_creator": "55ee82871d41c86ee1d78c45",
        "sign_off": "ADM",
        "media": "55fe4e691d41c8cac923ceb2",
        "source": "AAP",
        "original_source": "AAP Video/AAP",
        "pubstatus": "usable",
        "filemeta": {
            "mime_type": "video/mp4",
            "last_modification": "1904-01-01T00:00:00+00:00",
            "creation_date": "1904-01-01T00:00:00+00:00",
            "height": "270",
            "width": "480",
            "duration": "0:00:10.224000",
            "comment": "User volume: 100.0%",
            "length": 877869,
            "endian": "Big endian",
        },
        "event_id": "tag:localhost:2015:f3ae4441-4721-4987-8265-88d747b6a550",
        "original_creator": "55ee82871d41c86ee1d78c45",
        "expiry": "2016-12-21T14:14:11.000Z",
        "firstcreated": "2015-09-20T06:12:57.000Z",
        "_created": "2015-09-20T06:12:57.000Z",
        "type": "video",
        "unique_name": "#274",
        "mimetype": "video/mp4",
        "version": 2,
        "headline": "test video",
        "description": "test video",
        "abstract": "test video",
        "slugline": "test video keyword",
        "byline": "test video",
        "subject": [{"qcode": "01001000", "name": "archaeology", "parent": "01000000"}],
        "place": [{"qcode": "ACT", "name": "ACT"}],
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
    }

    vocab = [
        {
            "_id": "rightsinfo",
            "items": [
                {
                    "name": "AAP",
                    "copyrightHolder": "copy right holder",
                    "copyrightNotice": "copy right notice",
                    "usageTerms": "terms",
                },
                {
                    "name": "default",
                    "copyrightHolder": "default copy right holder",
                    "copyrightNotice": "default copy right notice",
                    "usageTerms": "default terms",
                },
            ],
        }
    ]

    package = {
        "_id": "urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701",
        "guid": "urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701",
        "type": "composite",
        "_current_version": 1,
        "groups": [
            {"role": "grpRole:NEP", "id": "root", "refs": [{"idRef": "main"}]},
            {
                "role": "grpRole:main",
                "id": "main",
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                        "_current_version": 4,
                    }
                ],
            },
        ],
        "pubstatus": "usable",
        "state": "published",
        "marked_for_not_publication": False,
        "dateline": {
            "located": {
                "alt_name": "",
                "state": "California",
                "city_code": "Los Angeles",
                "city": "Los Angeles",
                "dateline": "city",
                "country_code": "US",
                "country": "USA",
                "tz": "America/Los_Angeles",
                "state_code": "CA",
            },
            "date": "2015-08-12T01:59:58.000Z",
            "source": "AAP",
            "text": "Los Angeles, Aug 11 AAP -",
        },
        "language": "en",
        "headline": "Cop sacked over student shooting",
        "source": "AAP",
        "slugline": "US Police",
        "anpa_category": [{"name": "International News", "qcode": "I"}],
        "subject": [{"name": "police", "parent": "02000000", "qcode": "02003000"}],
    }

    picture_package = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                        "_current_version": 4,
                    }
                ],
                "id": "main",
                "role": "grpRole:main",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "marked_for_not_publication": False,
        "sign_off": "mar",
    }

    picture_text_package = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                        "_current_version": 4,
                    },
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                        "_current_version": 4,
                    },
                ],
                "id": "main",
                "role": "grpRole:main",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "marked_for_not_publication": False,
        "sign_off": "mar",
    }

    picture_text_package_multi_group = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}, {"idRef": "picture"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                        "_current_version": 4,
                    }
                ],
                "id": "main",
                "role": "grpRole:main",
            },
            {
                "refs": [
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                        "_current_version": 4,
                    }
                ],
                "id": "picture",
                "role": "grpRole:picture",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "marked_for_not_publication": False,
        "sign_off": "mar",
    }

    packaged_articles = [
        {
            "_id": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
            "headline": "package article headline",
            "slugline": "slugline",
            "_current_version": 4,
            "state": "published",
            "pubStatus": "usable",
        },
        {
            "_id": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
            "headline": "package article headline",
            "slugline": "slugline",
            "_current_version": 4,
            "state": "published",
            "pubStatus": "usable",
        },
    ]

    now = datetime.datetime(2015, 6, 13, 11, 45, 19, 0)

    def setUp(self):
        self.article["state"] = "published"
        self._setup_dates(
            [
                self.article,
                self.video,
                self.picture,
                self.package,
                self.picture_package,
                self.preformatted,
                self.picture_text_package,
                self.picture_text_package_multi_group,
            ]
        )
        self.newsml = etree.Element("NewsML")
        self.formatter = NewsML12Formatter()
        self.formatter.now = self.now
        self.formatter.string_now = self.now.strftime("%Y%m%dT%H%M%S+0000")
        with self.app.app_context():
            init_app(self.app)
            self.app.data.insert("vocabularies", self.vocab)

    def _setup_dates(self, item_list):
        for item in item_list:
            item["firstcreated"] = self.now
            item["versioncreated"] = self.now

    def test_format_news_envelope(self):
        self.formatter._format_news_envelope(self.article, self.newsml, 7)
        self.assertEqual(self.newsml.find("TransmissionId").text, "7")
        self.assertEqual(self.newsml.find("DateAndTime").text, "20150613T114519+0000")
        self.assertEqual(self.newsml.find("Priority").get("FormalName"), "1")
        newsml = etree.Element("NewsML")
        self.formatter._format_news_envelope(self.preformatted, newsml, 7)
        self.assertEqual(newsml.find("Priority").get("FormalName"), "5")

    def test_format_identification(self):
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/ProviderId").text, "sourcefabric.org")
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/DateId").text, "20150613")
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/NewsItemId").text, "urn:localhost.abc")
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/RevisionId").get("PreviousRevision"), "0")
        self.assertEqual(
            self.newsml.find("Identification/NewsIdentifier/PublicIdentifier").text, "urn:localhost.abc:5N"
        )
        self.assertEqual(self.newsml.find("Identification/DateLabel").text, "Saturday 13 June 2015")

    def test_format_identification_for_corrections(self):
        self.article["state"] = "corrected"
        self.article["_current_version"] = 7
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/RevisionId").get("PreviousRevision"), "6")
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/RevisionId").get("Update"), "N")
        self.article["state"] = "killed"
        self.formatter._format_identification(self.article, self.newsml)
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/RevisionId").get("PreviousRevision"), "6")
        self.assertEqual(self.newsml.find("Identification/NewsIdentifier/RevisionId").get("Update"), "N")

    def test_format_news_management(self):
        self.formatter._format_news_management(self.article, self.newsml)
        self.assertEqual(self.newsml.find("NewsManagement/NewsItemType").get("FormalName"), "News")
        self.assertEqual(self.newsml.find("NewsManagement/FirstCreated").text, "20150613T114519+0000")
        self.assertEqual(self.newsml.find("NewsManagement/ThisRevisionCreated").text, "20150613T114519+0000")
        self.assertEqual(self.newsml.find("NewsManagement/Status").get("FormalName"), "usable")
        self.assertEqual(self.newsml.find("NewsManagement/Urgency").get("FormalName"), "2")
        self.assertEqual(self.newsml.find("NewsManagement/Instruction").get("FormalName"), "Update")

    def test_format_news_management_for_corrections(self):
        self.article["state"] = "corrected"
        self.formatter._format_news_management(self.article, self.newsml)
        self.assertEqual(self.newsml.find("NewsManagement/Instruction").get("FormalName"), "Correction")

    def test_format_news_component(self):
        self.formatter._format_news_component(self.article, self.newsml)
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/Role").get("FormalName"), "Main")
        self.assertEqual(
            self.newsml.find("NewsComponent/NewsComponent/NewsLines/HeadLine").text,
            "This is a test headline - для теста!",
        )
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/NewsLines/ByLine").text, "joe")
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/NewsLines/DateLine").text, "sample dateline")
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/NewsLines/CreditLine").text, "AAP")
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/NewsLines/KeywordLine").text, "slugline")
        self.assertEqual(
            self.newsml.findall("NewsComponent/NewsComponent/DescriptiveMetadata/SubjectCode/Subject")[0].get(
                "FormalName"
            ),
            "02011001",
        )
        self.assertEqual(
            self.newsml.findall("NewsComponent/NewsComponent/DescriptiveMetadata/SubjectCode/Subject")[1].get(
                "FormalName"
            ),
            "02011002",
        )
        self.assertEqual(self.newsml.find("NewsComponent/NewsComponent/DescriptiveMetadata/Property").get("Value"), "a")
        self.assertEqual(
            self.newsml.findall("NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent")[0].text,
            "sample abstract",
        )
        self.assertEqual(
            self.newsml.findall(
                "NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent/nitf/body/body.content/p"
            )[0].text,
            "The story body\xa0more to the story - больше историй",
        )
        self.assertEqual(
            self.newsml.findall(
                "NewsComponent/NewsComponent/NewsComponent/ContentItem/DataContent/nitf/body/body.content/p"
            )[1].text,
            "call helpline 999 if you are planning to quit smoking",
        )
        self.assertEqual(self.newsml.find(".//NewsLines/NewsLine/NewsLineText").text, "this is test")

        company_info = self.newsml.find('NewsComponent/NewsComponent/Metadata/Property[@FormalName="Ticker Symbol"]')
        self.assertEqual(company_info.attrib["Value"], "YAL")

        company_info = self.newsml.find('NewsComponent/NewsComponent/Metadata/Property[@FormalName="Exchange"]')
        self.assertEqual(company_info.attrib["Value"], "ASX")

    def test_format_news_management_for_embargo(self):
        embargo_ts = utcnow() + datetime.timedelta(days=2)
        doc = self.article.copy()
        doc["embargo"] = embargo_ts

        self.formatter._format_news_management(doc, self.newsml)

        self.assertEqual(self.newsml.find("NewsManagement/NewsItemType").get("FormalName"), "News")
        self.assertEqual(self.newsml.find("NewsManagement/FirstCreated").text, "20150613T114519+0000")
        self.assertEqual(self.newsml.find("NewsManagement/ThisRevisionCreated").text, "20150613T114519+0000")
        self.assertEqual(self.newsml.find("NewsManagement/Urgency").get("FormalName"), "2")
        self.assertEqual(self.newsml.find("NewsManagement/Instruction").get("FormalName"), "Update")
        self.assertEqual(self.newsml.find("NewsManagement/Status").get("FormalName"), "Embargoed")
        self.assertEqual(self.newsml.find("NewsManagement/StatusWillChange/FutureStatus").get("FormalName"), "usable")
        self.assertEqual(self.newsml.find("NewsManagement/StatusWillChange/DateAndTime").text, embargo_ts.isoformat())

    def test_format_place(self):
        doc = self.article.copy()
        self.formatter._format_place(doc, self.newsml)
        self.assertEqual(
            self.newsml.find('Location/Property[@FormalName="CountryArea"]').get("Value"), "New South Wales"
        )
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="Country"]').get("Value"), "Australia")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="WorldRegion"]').get("Value"), "Oceania")

    def test_format_dateline(self):
        doc = self.article.copy()
        self.formatter._format_dateline(doc, self.newsml)
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="City"]').get("Value"), "Los Angeles")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="CountryArea"]').get("Value"), "California")
        self.assertEqual(self.newsml.find('Location/Property[@FormalName="Country"]').get("Value"), "USA")

    def test_duration(self):
        self.assertEqual(self.formatter._get_total_duration(None), 0)
        self.assertEqual(self.formatter._get_total_duration("dsf"), 0)
        self.assertEqual(self.formatter._get_total_duration("0:1:0.0000"), 60)
        self.assertEqual(self.formatter._get_total_duration("0:1:10.0000"), 70)
        self.assertEqual(self.formatter._get_total_duration("1:1:10.0000"), 3670)

    def test_format_picture(self):
        doc = self.picture.copy()
        seq, xml_str = self.formatter.format(doc, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(xml_str)

        self.assertEqual(
            xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/HeadLine").text, "NUS CHRISTOPHER PYNE PROTEST"
        )
        self.assertEqual(xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/ByLine").text, "TRACEY NEARMY")
        self.assertEqual(xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/CreditLine").text, "AAP Image/AAP")
        self.assertEqual(
            xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/KeywordLine").text, "NUS CHRISTOPHER PYNE PROTEST"
        )
        self.assertEqual(
            xml.find(
                ("NewsItem/NewsComponent/NewsComponent/DescriptiveMetadata/" 'Property[@FormalName="Category"]')
            ).get("Value"),
            "a",
        )

        for rendition, value in doc.get("renditions").items():
            xpath = './/Role[@FormalName="{}"]/../ContentItem'.format(rendition)
            content_item = xml.find(xpath)
            self.assertEqual(content_item.get("Href"), value.get("href"))
            self.assertEqual(content_item.find("MediaType").get("FormalName"), "Photo")
            self.assertEqual(content_item.find("Format").get("FormalName"), value.get("mimetype"))
            self.assertEqual(
                content_item.find('Characteristics/Property[@FormalName="Width"]').get("Value"), str(value.get("width"))
            )
            self.assertEqual(
                content_item.find('Characteristics/Property[@FormalName="Height"]').get("Value"),
                str(value.get("height")),
            )

    def test_format_video(self):
        doc = self.video.copy()
        seq, xml_str = self.formatter.format(doc, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/HeadLine").text, "test video")
        self.assertEqual(xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/ByLine").text, "test video")
        self.assertEqual(xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/CreditLine").text, "AAP Video/AAP")
        self.assertEqual(
            xml.find("NewsItem/NewsComponent/NewsComponent/NewsLines/KeywordLine").text, "test video keyword"
        )
        for rendition, value in doc.get("renditions").items():
            xpath = './/Role[@FormalName="{}"]/../ContentItem'.format(rendition)
            content_item = xml.find(xpath)
            self.assertEqual(content_item.get("Href"), value.get("href"))
            self.assertEqual(content_item.find("MediaType").get("FormalName"), "Video")
            self.assertEqual(content_item.find("Format").get("FormalName"), value.get("mimetype"))
            self.assertEqual(
                content_item.find('Characteristics/Property[@FormalName="Width"]').get("Value"),
                str(doc.get("filemeta", {}).get("width")),
            )
            self.assertEqual(
                content_item.find('Characteristics/Property[@FormalName="Height"]').get("Value"),
                str(doc.get("filemeta", {}).get("height")),
            )
            self.assertEqual(
                content_item.find('Characteristics/Property[@FormalName="TotalDuration"]').get("Value"), "10"
            )

    def test_format_package(self):
        doc = self.package.copy()
        seq, xml_str = self.formatter.format(doc, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(
            xml.find('.//Role[@FormalName="root"]/../NewsComponent/Role').get("FormalName"), "grpRole:main"
        )
        self.assertEqual(
            xml.find('.//Role[@FormalName="root"]/../NewsComponent/NewsItemRef').get("NewsItem"),
            "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b:4N",
        )

    def test_format_picture_package(self):
        doc = self.picture_package.copy()
        seq, xml_str = self.formatter.format(doc, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(xml_str)
        self.assertEqual(
            xml.find('.//Role[@FormalName="root"]/../NewsComponent/Role').get("FormalName"), "grpRole:main"
        )
        self.assertEqual(
            xml.find('.//Role[@FormalName="root"]/../NewsComponent/NewsItemRef').get("NewsItem"),
            "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037:4N",
        )

    def test_format_picture_text_package(self):
        doc = self.picture_text_package.copy()
        seq, xml_str = self.formatter.format(doc, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(xml_str)
        news_component = xml.find('.//Role[@FormalName="root"]/../NewsComponent')
        self.assertEqual(news_component.find("Role").get("FormalName"), "grpRole:main")
        news_item_refs = news_component.findall("NewsItemRef")
        self.assertEqual(
            news_item_refs[0].get("NewsItem"), "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b:4N"
        )
        self.assertEqual(
            news_item_refs[1].get("NewsItem"), "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037:4N"
        )



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2018-04-04 16:55

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "archive"  # will use multiple resources, keeping this here so validation passes

    def forwards(self, mongodb_collection, mongodb_database):
        for resource in ["archive", "archive_autosave", "published"]:

            collection = mongodb_database[resource]

            for item in collection.find({"editor_state": {"$exists": True}}):
                state = item["editor_state"]
                fields_meta = {"body_html": {"draftjsState": state}}
                print(
                    collection.update(
                        {"_id": item["_id"]}, {"$set": {"fields_meta": fields_meta}, "$unset": {"editor_state": ""}}
                    )
                )

    def backwards(self, mongodb_collection, mongodb_database):
        for resource in ["archive", "archive_autosave", "published"]:

            collection = mongodb_database[resource]

            for item in collection.find({"fields_meta": {"$exists": True}}):
                state = item["fields_meta"]["body_html"]["draftjsState"]
                print(
                    collection.update(
                        {"_id": item["_id"]}, {"$set": {"editor_state": state}, "$unset": {"fields_meta": ""}}
                    )
                )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from enum import Enum, unique


@unique
class ConceptNature(Enum):
    """
    Concept items types agreed by the IPTC

    http://cv.iptc.org/newscodes/cpnature/
    """

    ABSTRACT = "cpnat:abstract"
    EVENT = "cpnat:event"
    GEOAREA = "cpnat:geoArea"
    OBJECT = "cpnat:object"
    ORGANISATION = "cpnat:organisation"
    PERSON = "cpnat:person"
    POI = "cpnat:poi"

    @classmethod
    def values(cls):
        return [i.value for i in cls]



# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import logging
import requests
import superdesk

from flask import current_app
from collections import OrderedDict
from typing import Optional, Dict, List
from urllib.parse import urljoin
from superdesk.text_utils import get_text
from superdesk.errors import SuperdeskApiError
from .base import AIServiceBase

logger = logging.getLogger(__name__)
TIMEOUT = 30

# iMatrics concept type to SD type mapping
CONCEPT_MAPPING = OrderedDict(
    [
        # following concepts don't have clear equivalent in SD
        ("category", "subject"),
        ("object", "object"),
        ("entity", "organisation"),
        ("event", "event"),
        ("topic", "subject"),
        ("organisation", "organisation"),
        ("Name LastName", "person"),
        # both Name LastName and person are seens in iMatrics examples and docs
        ("person", "person"),
        ("place", "place"),
    ]
)

DEFAULT_CONCEPT_TYPE = "topic"


class IMatrics(AIServiceBase):
    """IMatrics autotagging service

    The IMATRICS_BASE_URL, IMATRICS_USER and IMATRICS_KEY setting (or environment variable) must be set
    IMATRICS_AUTHOR can be used to set ``author`` of concept (which translates to ``source`` in Superdesk)
    """

    name = "imatrics"
    label = "IMatrics autotagging service"

    def __init__(self, app):
        super().__init__(app)
        self.convept_map_inv = {v: k for k, v in CONCEPT_MAPPING.items()}
        self._subjects = []

    @property
    def base_url(self):
        return current_app.config.get("IMATRICS_BASE_URL", os.environ.get("IMATRICS_BASE_URL"))

    @property
    def user(self):
        return current_app.config.get("IMATRICS_USER", os.environ.get("IMATRICS_USER"))

    @property
    def key(self):
        return current_app.config.get("IMATRICS_KEY", os.environ.get("IMATRICS_KEY"))

    def concept2tag_data(self, concept: dict) -> dict:
        """Convert an iMatrics concept to Superdesk friendly data"""
        tag_data = {
            "name": concept["title"],
            "qcode": concept["uuid"],
            "source": "imatrics",
            "altids": {
                "imatrics": concept["uuid"],
            },
        }

        if concept.get("shortDescription") and concept["shortDescription"].strip():
            tag_data["description"] = concept["shortDescription"].strip()

        try:
            tag_type = CONCEPT_MAPPING[concept["type"]]
        except KeyError:
            logger.warning("no mapping for concept type {concept_type!r}".format(concept_type=concept["type"]))
            tag_type = concept["type"]

        tag_data["type"] = tag_type

        try:
            tag_data["weight"] = concept["weight"]
        except KeyError:
            pass

        for link in concept.get("links", []):
            if link.get("source") == "IPTC":
                topic_id = link.get("id", "")
                if topic_id.startswith("medtop:"):
                    topic_id = topic_id[7:]
                    subject = self.find_subject(topic_id)
                    if subject:
                        tag_data.update(subject)
                    tag_data["altids"]["medtop"] = topic_id

        if concept["type"] in ("topic", "category"):
            tag_data.setdefault("scheme", "imatrics_{}".format(concept["type"]))

        return tag_data

    def find_subject(self, topic_id):
        SCHEME_ID = current_app.config.get("IMATRICS_SUBJECT_SCHEME")
        if not SCHEME_ID:
            return
        if not self._subjects:
            cv = superdesk.get_resource_service("vocabularies").find_one(req=None, _id=SCHEME_ID)
            if cv and cv.get("items"):
                self._subjects = [item for item in cv["items"] if item.get("is_active")]
        for subject in self._subjects:
            if subject.get("qcode") == topic_id:
                return superdesk.get_resource_service("vocabularies").get_article_cv_item(subject, SCHEME_ID)

    def check_verb(self, expected: str, verb: str, operation: str) -> None:
        """Check that HTTP verb use is the one expected for this operation"""
        if verb != expected:
            raise SuperdeskApiError.badRequestError(
                "[{name}] Unexpected verb for {operation}: {verb}".format(
                    name=self.name, verb=verb, operation=operation
                )
            )

    def analyze(self, item: dict) -> dict:
        """Analyze article to get tagging suggestions"""
        if not self.base_url or not self.user or not self.key:
            logger.warning("IMatrics is not configured propertly, can't analyze article")
            return {}
        body = get_item_body(item)
        headline = item.get("headline", "")
        if not body and not headline:
            logger.warning("no body nor headline found in item {item_id!r}".format(item_id=item["guid"]))
            # we return an empty result
            return {"subject": []}

        data = {
            "uuid": item["guid"],
            "pubStatus": False,
            "headline": headline,
            "body": body,
        }

        r_data = self._request("article/concept", data)

        analyzed_data: Dict[str, List] = {}

        for concept in r_data:
            tag_data = self.concept2tag_data(concept)
            tag_type = tag_data.pop("type")
            analyzed_data.setdefault(tag_type, []).append(tag_data)

        for tags in analyzed_data.values():
            tags.sort(key=lambda d: d["weight"], reverse=True)
            for tag in tags:
                del tag["weight"]

        return analyzed_data

    def search(self, data: dict) -> dict:

        data = {
            "title": data["term"],
            "type": "all",
            "draft": False,
            "size": 10,
        }

        r_data = self._request("concept/get", data, params=dict(operation="title_type"))

        tags: Dict[str, List[Dict]] = {}
        ret = {"tags": tags}
        for concept in r_data["result"]:
            tag_data = self.concept2tag_data(concept)
            tag_type = tag_data.pop("type")
            tags.setdefault(tag_type, []).append(tag_data)

        return ret

    def create(self, data: dict) -> dict:
        concept = {}

        try:
            concept["title"] = data["title"]
        except KeyError:
            raise SuperdeskApiError.badRequestError(
                "[{name}] missing title when creating tag: {data}".format(name=self.name, data=data)
            )

        sd_type = data.get("type", "subject")
        try:
            concept["type"] = self.convept_map_inv[sd_type]
        except KeyError:
            logger.warning("no mapping for superdesk type {sd_type!r}".format(sd_type=sd_type))
            concept["type"] = "topic"

        r_data = self._request("concept/create", concept)

        if r_data["error"]:
            raise SuperdeskApiError.proxyError(
                "iMatrics concept creation failed: {msg}".format(msg=r_data.get("response", ""))
            )

        return {}

    def delete(self, data: dict) -> dict:
        try:
            uuid = data["uuid"].strip()
            if not uuid:
                raise KeyError
        except KeyError:
            raise SuperdeskApiError.badRequestError("[{name}] no tag UUID specified".format(name=self.name))

        self._request("concept/delete", method="DELETE", params={"uuid": data["uuid"]})
        return {}

    def data_operation(self, verb: str, operation: str, name: Optional[str], data: dict) -> dict:
        if not self.base_url or not self.user or not self.key:
            logger.warning("IMatrics is not configured propertly, can't analyze article")
            return {}
        if operation == "search":
            self.check_verb("POST", verb, operation)
            return self.search(data)
        elif operation == "create":
            self.check_verb("POST", verb, operation)
            return self.create(data)
        elif operation == "delete":
            self.check_verb("POST", verb, operation)
            return self.delete(data)
        else:
            raise SuperdeskApiError.badRequestError(
                "[{name}] Unexpected operation: {operation}".format(name=name, operation=operation)
            )

    def publish(self, data):
        return self._request("article/concept", data)

    def _request(self, service, data=None, method="POST", params=None):
        url = urljoin(self.base_url, service)
        r = requests.request(method, url, json=data, auth=(self.user, self.key), params=params, timeout=TIMEOUT)
        if r.status_code != 200:
            raise SuperdeskApiError.proxyError(
                "Unexpected return code ({status_code}) from {name}: {msg}".format(
                    name=self.name,
                    status_code=r.status_code,
                    msg=r.text,
                )
            )
        return r.json()


def get_item_body(item):
    body = []
    for field in ("body_html", "abstract"):
        try:
            body.extend([p.strip() for p in get_text(item[field], "html", True).split("\n") if p.strip()])
        except KeyError:
            pass
    return body



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.auth.service import AuthService
from superdesk import get_resource_service
from superdesk.resource import Resource
from apps.auth.errors import CredentialsAuthError
from superdesk.errors import SuperdeskApiError
from superdesk import utils
from flask import current_app as app
import requests
import superdesk


class XMPPAuthResource(Resource):
    schema = {
        "jid": {"type": "string", "required": True},
        "transactionId": {"type": "string", "required": True},
        "token": {"type": "string"},
        "user": Resource.rel("users", True),
    }
    resource_methods = ["POST"]
    public_methods = ["POST"]
    extra_response_fields = ["user", "token", "username"]


superdesk.intrinsic_privilege("auth_xmpp", method=["DELETE"])


class XMPPAuthService(AuthService):
    def authenticate(self, credentials):
        auth_url = app.config["XMPP_AUTH_URL"]
        if not auth_url:
            raise SuperdeskApiError.notConfiguredError()
        domain = app.config["XMPP_AUTH_DOMAIN"]
        jid = credentials.get("jid")
        if not jid:
            raise CredentialsAuthError(credentials)
        user = get_resource_service("auth_users").find_one(req=None, jid=jid)
        if not user:
            raise CredentialsAuthError(credentials)

        try:
            r = requests.post(
                app.config["XMPP_AUTH_URL"],
                data={"jid": jid, "domain": domain, "transaction_id": credentials.get("transactionId")},
            )
        except Exception:
            raise CredentialsAuthError(credentials)
        else:
            if r.status_code != 200:
                raise CredentialsAuthError(credentials)

        return user

    def set_auth_default(self, doc, user_id):
        doc["user"] = user_id
        doc["token"] = utils.get_random_string(40)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from unittest import TestCase
from superdesk.media.image import get_meta
from superdesk.media.media_operations import crop_image


fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), "fixtures")


class ExifMetaExtractionTestCase(TestCase):

    img = os.path.join(fixtures, "canon_exif.JPG")

    def test_extract_meta_json_serialization(self):

        with open(self.img, mode="rb") as f:
            meta = get_meta(f)

        self.assertEqual(meta["ExifImageWidth"], 32)
        self.assertEqual(meta["ExifImageHeight"], 21)
        self.assertEqual(meta["Make"], "Canon")
        self.assertEqual(meta["Model"], "Canon EOS 60D")


class ExifMetaWithGPSInfoExtractionTestCase(TestCase):

    img = os.path.join(fixtures, "iphone_gpsinfo_exif.JPG")
    maxDiff = None

    def test_extract_meta_json_serialization(self):
        expected_gpsinfo = {
            "GPSImgDirection": 31.145,
            "GPSLongitudeRef": "W",
            "GPSImgDirectionRef": "T",
            "GPSAltitudeRef": 0,
            "GPSLatitudeRef": "S",
            "GPSAltitude": 576.978,
            "GPSLatitude": (33.0, 26.0, 11.5),
            "GPSLongitude": (70.0, 38.0, 39.46),
            "GPSTimeStamp": (19.0, 59.0, 51.17),
        }

        with open(self.img, mode="rb") as f:
            meta = get_meta(f)

        self.assertEqual(meta.get("ExifImageWidth", None), 400)
        self.assertEqual(meta.get("ExifImageHeight", None), 300)
        self.assertEqual(meta.get("Make", None), "Apple")
        self.assertEqual(meta.get("Model", None), "iPhone 5")
        self.assertEqual(meta.get("GPSInfo", None), expected_gpsinfo)


class ExifMetaExtractionUserCommentRemovedTestCase(TestCase):

    # this image has UserComment exif data value as 'test'
    img = os.path.join(fixtures, "iphone_gpsinfo_exif.JPG")

    def test_extract_meta_json_serialization(self):

        with open(self.img, mode="rb") as f:
            meta = get_meta(f)

        self.assertIsNone(meta.get("UserComment", None))


class MediaOperationsTestCase(TestCase):

    img = os.path.join(fixtures, "canon_exif.JPG")

    crop_data = {
        "CropLeft": 1,
        "CropTop": 1,
        "CropRight": 9,
        "CropBottom": 7,
    }

    def test_crop_image(self):
        with open(self.img, mode="rb") as f:
            status, output = crop_image(f, "test", self.crop_data)
            self.assertEqual(True, status)
            self.assertEqual(8, output.width)
            self.assertEqual(6, output.height)

    def test_crop_image_resize(self):
        with open(self.img, mode="rb") as f:
            status, output = crop_image(f, "test", self.crop_data, {"width": 4, "height": 3})
            self.assertEqual(True, status)
            self.assertEqual(4, output.width)
            self.assertEqual(3, output.height)



from superdesk.tests import TestCase
from apps.archive.commands import RemoveExpiredContent


class RemoveExpiredContentTestCase(TestCase):
    def test_is_expired(self):
        command = RemoveExpiredContent()
        item = {"expiry": None, "_id": "foo"}
        self.assertFalse(command._can_remove_item(item))



from superdesk.tests import TestCase
from superdesk.vocabularies.commands import get_vocabularies, is_changed, update_item


class VocabulariesUpdateArchiveTestCase(TestCase):
    fields = ["subject", "genre", "place", "anpa_category"]
    vocabularies_list = [
        {
            "_id": "country_custom",
            "items": [
                {
                    "name": "Australia",
                    "qcode": "country_custom:1001001",
                    "is_active": True,
                    "translations": {"name": {"de": "Australien", "it": "Australia", "ja": "オーストラリア"}},
                },
                {
                    "name": "Austria",
                    "qcode": "country_custom:1001002",
                    "is_active": True,
                    "translations": {"name": {"de": "Österreich", "it": "Austria", "ja": "オーストリア"}},
                },
            ],
        },
        {
            "_id": "subject_custom",
            "items": [
                {
                    "name": "Asset allocation",
                    "parent": None,
                    "is_active": True,
                    "qcode": "subject:01000000",
                    "translations": {
                        "name": {"de": "Asset-Allokation", "it": "Asset allocation", "ja": "アセット・アロケーション"}
                    },
                },
                {
                    "name": "Risk",
                    "parent": None,
                    "is_active": True,
                    "qcode": "subject:02000000",
                    "translations": {"name": {"de": "Risiko", "it": "Rischio", "ja": "リスク"}},
                },
                {
                    "name": "Market context",
                    "parent": None,
                    "is_active": True,
                    "qcode": "subject:03000000",
                    "translations": {"name": {"de": "Marktumfeld", "it": "Contesto di mercato", "ja": "マーケット・コンテクスト"}},
                },
            ],
        },
    ]

    def test_get_vocabularies(self):
        vocabularies = get_vocabularies(self.vocabularies_list)

        self.assertTrue("country_custom" in vocabularies)
        self.assertTrue("values" in vocabularies["country_custom"])
        self.assertTrue("country_custom:1001001" in vocabularies["country_custom"]["values"])
        self.assertTrue("country_custom:1001002" in vocabularies["country_custom"]["values"])
        self.assertTrue(
            vocabularies["country_custom"]["values"]["country_custom:1001001"]["scheme"] == "country_custom"
        )
        self.assertTrue(
            vocabularies["country_custom"]["values"]["country_custom:1001002"]["scheme"] == "country_custom"
        )

        self.assertTrue("subject_custom" in vocabularies)
        self.assertTrue("values" in vocabularies["subject_custom"])
        self.assertTrue("values" in vocabularies["subject_custom"])
        self.assertTrue("subject:01000000" in vocabularies["subject_custom"]["values"])
        self.assertTrue("subject:02000000" in vocabularies["subject_custom"]["values"])
        self.assertTrue("subject:03000000" in vocabularies["subject_custom"]["values"])
        self.assertTrue(vocabularies["subject_custom"]["values"]["subject:01000000"]["scheme"] == "subject_custom")
        self.assertTrue(vocabularies["subject_custom"]["values"]["subject:02000000"]["scheme"] == "subject_custom")
        self.assertTrue(vocabularies["subject_custom"]["values"]["subject:03000000"]["scheme"] == "subject_custom")

    def test_is_changed(self):
        old = {
            "name": "Market context",
            "parent": None,
            "qcode": "subject:03000000",
            "translations": {"name": {"de": "Marktumfeld", "it": "Contesto di mercato", "ja": "マーケット・コンテクスト"}},
        }

        new = {
            "name": "Market context",
            "parent": None,
            "qcode": "subject:03000000",
            "translations": {"name": {"de": "changed", "it": "Contesto di mercato", "ja": "マーケット・コンテクスト"}},
        }

        new_no_translations = {"name": "Market context", "parent": None, "qcode": "subject:03000000"}

        self.assertTrue(is_changed(old, {}))
        self.assertTrue(is_changed(old, {"name": "Market context"}))
        self.assertTrue(is_changed(old, new_no_translations))
        self.assertFalse(is_changed(old, old))
        self.assertTrue(is_changed(old, new))

    def test_update_item(self):
        vocabularies = get_vocabularies(self.vocabularies_list)

        item = {
            "place": [self.vocabularies_list[0]["items"][0], self.vocabularies_list[0]["items"][0]],
            "subject": [self.vocabularies_list[1]["items"][1], self.vocabularies_list[1]["items"][1]],
        }

        item["subject"][0]["scheme"] = "country_custom"
        item["subject"][0]["scheme"] = "country_custom"
        item["place"][0]["scheme"] = "subject_custom"
        item["place"][0]["scheme"] = "subject_custom"
        changed = {"qcode": "subject:03000000", "scheme": "subject_custom"}
        item["subject"].append(changed)

        updates = update_item(item, vocabularies, self.fields)

        self.assertTrue("subject" in updates)
        self.assertTrue(updates["subject"][0].get("qcode") == "subject:03000000")
        self.assertTrue("translations" in updates["subject"][0])



import io
import time
import superdesk

from bson import ObjectId
from copy import copy
from flask import json, request
from datetime import timedelta
from superdesk.tests import TestCase
from superdesk.utc import utcnow
from content_api.publish import MONGO_PREFIX
from content_api.app import get_app
from eve.utils import ParsedRequest
from eve.methods.common import store_media_files
from werkzeug.datastructures import MultiDict


class ContentAPITestCase(TestCase):
    def setUp(self):
        self.content_api = superdesk.get_resource_service("content_api")
        self.db = self.app.data.mongo.pymongo(prefix=MONGO_PREFIX).db
        self.app.config["SECRET_KEY"] = "secret"
        config = copy(self.app.config)
        config["AMAZON_CONTAINER_NAME"] = None  # force gridfs
        config["URL_PREFIX"] = ""
        config["MEDIA_PREFIX"] = "/assets"
        self.capi = get_app(config)
        self.capi.testing = True
        self.subscriber = {"_id": "sub1"}

    def _auth_headers(self, sub=None):
        if sub is None:
            sub = self.subscriber
        service = superdesk.get_resource_service("subscriber_token")
        payload = {"subscriber": sub.get("_id")}
        service.create([payload])
        token = payload["_id"]
        headers = {"Authorization": "Token " + token}
        return headers

    def test_publish_to_content_api(self):
        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}, "rewrite_of": "bar"}
        self.content_api.publish(item)
        self.assertEqual(1, self.db.items.count())
        self.assertNotIn("task", self.db.items.find_one())
        self.assertEqual("foo", self.db.items.find_one()["_id"])

        item["_current_version"] = "2"
        self.content_api.publish(item)
        self.assertEqual(1, self.db.items.count())

        item["_current_version"] = "3"
        item["headline"] = "foo"
        self.content_api.publish(item)
        self.assertEqual("foo", self.db.items.find_one()["headline"])
        self.assertEqual("bar", self.db.items.find_one()["evolvedfrom"])

        self.assertEqual(3, self.db.items_versions.count())

    def test_create_keeps_planning_metadata(self):
        item = {
            "guid": "foo",
            "type": "text",
            "planning_id": "planning-id",
            "coverage_id": "coverage-id",
            "agenda_id": "agenda-id",
        }
        self.content_api.create([item])
        self.assertEqual(item["planning_id"], self.db.items.find_one()["planning_id"])
        self.assertEqual(item["coverage_id"], self.db.items.find_one()["coverage_id"])
        self.assertEqual(item["agenda_id"], self.db.items.find_one()["agenda_id"])

    def test_publish_with_subscriber_ids(self):
        item = {"guid": "foo", "type": "text"}
        subscribers = [{"_id": ObjectId()}, {"_id": ObjectId()}]

        self.content_api.publish(item, subscribers)
        self.assertEqual(1, self.db.items.find({"subscribers": str(subscribers[0]["_id"])}).count())
        self.assertEqual(0, self.db.items.find({"subscribers": "foo"}).count())

    def test_content_filtering_by_subscriber(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        self.content_api.publish({"_id": "foo", "guid": "foo", "type": "text"}, [subscriber])
        self.content_api.publish({"_id": "bar", "guid": "bar", "type": "text"}, [])
        self.content_api.publish({"_id": "pkg", "guid": "pkg", "type": "composite"}, [subscriber])
        self.content_api.publish({"_id": "pkg2", "guid": "pkg2", "type": "composite"}, [])

        with self.capi.test_client() as c:
            response = c.get("items")
            self.assertEqual(401, response.status_code)
            response = c.get("items", headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertEqual(1, len(data["_items"]))
            self.assertNotIn("subscribers", data["_items"][0])
            self.assertIn("items/foo", data["_items"][0]["uri"])

            audit_entries = superdesk.get_resource_service("api_audit").get(req=None, lookup={})
            self.assertEqual(1, audit_entries.count())

            response = c.get("packages", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, len(data["_items"]))
            self.assertIn("packages/pkg", data["_items"][0]["uri"])

            audit_entries = superdesk.get_resource_service("api_audit").get(req=None, lookup={})
            self.assertEqual(2, audit_entries.count())

    def test_content_filtering_by_arguments(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        self.content_api.publish({"_id": "foo", "guid": "foo", "urgency": "3", "type": "text"}, [subscriber])
        self.content_api.publish({"_id": "bar", "guid": "bar", "urgency": "4", "type": "text"}, [subscriber])

        with self.capi.test_client() as c:
            response = c.get("items")
            self.assertEqual(401, response.status_code)
            response = c.get("items", headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertEqual(2, len(data["_items"]))

            response = c.get('items?filter=[{"term":{"urgency": 3}}]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertEqual(1, len(data["_items"]))

    def test_content_api_picture(self):
        self.content_api.publish(
            {
                "_id": "foo",
                "guid": "foo",
                "type": "picture",
                "headline": "foo",
                "renditions": {
                    "original": {
                        "media": "abcd1234",
                        "width": 300,
                        "height": 200,
                        "mimetype": "image/jpeg",
                        "href": "foo",
                    }
                },
            }
        )

        headers = self._auth_headers()

        with self.capi.test_client() as c:
            response = c.get("items/foo", headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertIn("renditions", data)
            rendition = data["renditions"]["original"]
            self.assertNotIn("media", rendition)
            self.assertIn("assets/abcd1234", rendition["href"])

            response = c.get(rendition["href"])
            self.assertEqual(401, response.status_code)

            response = c.get(rendition["href"], headers=headers)
            self.assertEqual(404, response.status_code)

            with self.app.app_context():
                data = io.BytesIO(b"content")
                media_id = self.app.media.put(data, resource="upload")
                self.assertIsInstance(media_id, ObjectId, media_id)

            url = "assets/%s" % media_id
            response = c.get(url, headers=headers)
            self.assertEqual(200, response.status_code, url)
            self.assertEqual(b"content", response.data)

            audit_entries = superdesk.get_resource_service("api_audit").get(req=None, lookup={"type": "asset"})
            self.assertEqual(1, audit_entries.count())

    def test_text_with_pic_associations(self):
        subscriber = {"_id": "sub1"}
        self.content_api.publish(
            {
                "guid": "text",
                "type": "text",
                "body_html": """<p>
            <p>hey</p>
            <!-- EMBED START Image {id: \"foo\"} -->
            <figure>
                <img src=\"http://localhost:5000/api/upload/foo/raw?_schema=http\"
                    alt=\"tractor\"
                    srcset=\"//localhost:5000/api/upload/foo/raw?_schema=http 800w\" />
                <figcaption>tractor</figcaption>
            </figure>
            <!-- EMBED END Image {id: \"embedded12554054581\"} -->
            """,
                "associations": {
                    "foo": {
                        "type": "picture",
                        "renditions": {
                            "original": {
                                "href": "http://localhost:5000/api/upload/foo/raw?_schema=http",
                                "media": "bar",
                            }
                        },
                        "subscribers": ["sub1"],
                    },
                },
            },
            [subscriber],
        )

        headers = self._auth_headers(subscriber)
        with self.capi.test_client() as c:
            response = c.get("items/text", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, len(data["associations"]))
            renditions = data["associations"]["foo"]["renditions"]
            self.assertIn("assets/bar", renditions["original"]["href"])
            self.assertNotIn("http://localhost:5000/api/upload/", data["body_html"])

    def test_association_reset(self):
        subscriber = {"_id": "sub1"}
        self.content_api.publish(
            {
                "guid": "text",
                "type": "text",
                "body_html": "",
                "version": 1,
                "associations": {
                    "foo": {
                        "guid": "foo",
                        "type": "text",
                        "body_html": "<p>Foo</p>",
                    },
                },
            },
            [subscriber],
        )

        self.content_api.publish(
            {
                "guid": "text",
                "type": "text",
                "body_html": "updated",
                "version": 2,
                "assocations": {
                    "foo": None,
                },
            }
        )

        headers = self._auth_headers(subscriber)
        with self.capi.test_client() as c:
            response = c.get("items/text", headers=headers)
            data = json.loads(response.data)
            self.assertEqual({}, data["associations"])

    def test_content_filtering(self):
        self.content_api.publish({"guid": "u3", "type": "text", "source": "foo", "urgency": 3}, [self.subscriber])
        self.content_api.publish({"guid": "u2", "type": "text", "source": "bar", "urgency": 2}, [self.subscriber])

        headers = self._auth_headers()

        with self.capi.test_client() as c:
            response = c.get('items?where={"urgency":3}', headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, data["_meta"]["total"])
            self.assertEqual(3, data["_items"][0]["urgency"])

            response = c.get("items?q=urgency:3", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, data["_meta"]["total"])
            self.assertEqual(3, data["_items"][0]["urgency"])

            response = c.get("items?urgency=3", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, data["_meta"]["total"])
            self.assertEqual(3, data["_items"][0]["urgency"])

            response = c.get("items?urgency=[3,2]", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(2, data["_meta"]["total"])

            response = c.get("items?item_source=foo", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, data["_meta"]["total"])
            self.assertEqual("foo", data["_items"][0]["source"])

            response = c.get('items?item_source=["foo","bar"]', headers=headers)
            data = json.loads(response.data)
            self.assertEqual(2, data["_meta"]["total"])

    def test_generate_token_service(self):
        service = superdesk.get_resource_service("subscriber_token")
        payload = {"subscriber": "foo", "expiry_days": 7}
        ids = service.create([payload])
        token = payload["_id"]
        self.assertEqual("foo", self.capi.auth.check_auth(token, [], "items", "get"))
        self.assertLessEqual((utcnow() + timedelta(days=7)).timestamp() - payload["expiry"].timestamp(), 1)

        service.delete({"_id": ids[0]})
        self.assertFalse(self.capi.auth.check_auth(token, [], "items", "get"))

        payload = {"subscriber": "foo", "expiry": utcnow() - timedelta(days=1)}
        service.create([payload])
        token = payload["_id"]
        self.assertFalse(self.capi.auth.check_auth(token, [], "items", "get"))
        self.assertIsNone(service.find_one(None, _id=token))

    def test_api_block(self):
        self.app.data.insert(
            "filter_conditions", [{"_id": 1, "operator": "eq", "field": "source", "value": "fred", "name": "Fred"}]
        )
        content_filter = {
            "_id": 1,
            "name": "fred API Block",
            "content_filter": [{"expression": {"fc": [1]}}],
            "api_block": True,
        }
        self.app.data.insert("content_filters", [content_filter])

        self.content_api.publish({"_id": "foo", "source": "fred", "type": "text", "guid": "foo"})
        self.assertEqual(0, self.db.items.count())
        self.content_api.publish({"_id": "bar", "source": "jane", "type": "text", "guid": "bar"})
        self.assertEqual(1, self.db.items.count())

    def test_item_versions_api(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}, "rewrite_of": "bar", "_current_version": 1}
        self.content_api.publish(item, [subscriber])
        item["_current_version"] = 2
        self.content_api.publish(item, [subscriber])
        item["_current_version"] = 3
        self.content_api.publish(item, [subscriber])

        with self.capi.test_client() as c:
            response = c.get("items/foo?version=all", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(3, data["_meta"]["total"])

            response = c.get("items/foo?version=2", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(str(2), data["version"])

    def test_package_version(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        item = {"_id": "pkg", "guid": "pkg", "type": "composite", "_current_version": 1}
        self.content_api.publish(item, [subscriber])
        item["_current_version"] = 2
        self.content_api.publish(item, [subscriber])
        item["_current_version"] = 3
        self.content_api.publish(item, [subscriber])
        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}, "rewrite_of": "bar", "_current_version": 1}
        self.content_api.publish(item, [subscriber])

        with self.capi.test_client() as c:
            response = c.get("packages", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, len(data["_items"]))
            self.assertIn("packages/pkg", data["_items"][0]["uri"])

            response = c.get("packages/pkg?version=all", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(3, data["_meta"]["total"])

            response = c.get("packages/pkg?version=2", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(str(2), data["version"])

    def test_publish_kill_to_content_api(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)
        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}, "rewrite_of": "bar", "pubstatus": "usable"}
        self.content_api.publish(item, [subscriber])

        with self.capi.test_client() as c:
            response = c.get("items/foo?version=all", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(1, data["_meta"]["total"])

            response = c.get("items/foo?version=1", headers=headers)
            data = json.loads(response.data)
            self.assertEqual("1", data["version"])

        item["pubstatus"] = "canceled"
        item["_current_version"] = 2
        self.content_api.publish(item, [subscriber])

        self.assertEqual(1, self.db.items.count())
        self.assertEqual("canceled", self.db.items.find_one()["pubstatus"])
        self.assertEqual(2, self.db.items_versions.count())
        for i in self.db.items_versions.find():
            self.assertEqual(i.get("pubstatus"), "canceled")

        with self.capi.test_client() as c:
            response = c.get("items/foo?version=all", headers=headers)
            data = json.loads(response.data)
            self.assertEqual(0, data["_meta"]["total"])

            response = c.get("items/foo?version=1", headers=headers)
            self.assertEqual(404, response._status_code)

    def test_publish_item_with_ancestors(self):
        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}, "bookmarks": [ObjectId()]}
        self.content_api.publish(item)
        self.assertEqual(1, self.db.items.count())
        self.assertNotIn("ancestors", self.db.items.find_one({"_id": "foo"}))

        item["guid"] = "bar"
        item["rewrite_of"] = "foo"
        self.content_api.publish(item)

        self.assertEqual(2, self.db.items.count())
        bar = self.db.items.find_one({"_id": "bar"})
        self.assertEqual(["foo"], bar.get("ancestors", []))
        self.assertEqual("foo", bar.get("evolvedfrom"))
        foo = self.db.items.find_one({"_id": "foo"})
        self.assertEqual("bar", foo["nextversion"])

        item["guid"] = "fun"
        item["rewrite_of"] = "bar"
        self.content_api.publish(item)

        self.assertEqual(3, self.db.items.count())
        fun = self.db.items.find_one({"_id": "fun"})
        self.assertEqual(["foo", "bar"], fun.get("ancestors", []))
        self.assertEqual("bar", fun.get("evolvedfrom"))

    def test_sync_bookmarks_on_publish(self):
        item = {"guid": "foo", "type": "text", "task": {"desk": "foo"}}
        self.content_api.publish(item)
        self.db.items.update_one({"_id": "foo"}, {"$set": {"bookmarks": [ObjectId()]}})

        item["guid"] = "bar"
        item["rewrite_of"] = "foo"
        self.content_api.publish(item)

        bar = self.db.items.find_one({"_id": "bar"})
        self.assertEqual(1, len(bar["bookmarks"]))

    def test_search_capi(self):
        subscriber = {"_id": "sub1"}

        self.content_api.publish(
            {
                "_id": "foo",
                "guid": "foo",
                "type": "text",
                "anpa_category": [{"qcode": "i", "name": "International News"}],
                "headline": "Man bites dog",
            },
            [subscriber],
        )
        self.content_api.publish({"_id": "bar", "guid": "bar", "type": "text"}, [{"_id": "sub2"}])

        test = superdesk.get_resource_service("search_capi")
        req = ParsedRequest()
        req.args = MultiDict([("subscribers", "sub1")])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 1)

        resp = test.get(req=None, lookup=None)
        self.assertEqual(resp.count(), 2)

        req = ParsedRequest()
        req.args = MultiDict()
        req.where = '{"headline":"dog"}'
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 1)

    def test_search_capi_filter(self):
        subscriber = {"_id": "sub1"}

        self.content_api.publish(
            {
                "_id": "foo",
                "guid": "foo",
                "type": "text",
                "anpa_category": [{"qcode": "i", "name": "International News"}],
                "headline": "Man bites dog",
            },
            [subscriber],
        )
        self.content_api.publish(
            {
                "_id": "bar",
                "guid": "bar",
                "anpa_category": [{"qcode": "i", "name": "International News"}],
                "type": "text",
            },
            [{"_id": "sub2"}],
        )
        self.content_api.publish(
            {"_id": "nat", "guid": "nat", "anpa_category": [{"qcode": "a", "name": "National News"}], "type": "text"},
            [{"_id": "sub2"}],
        )

        test = superdesk.get_resource_service("search_capi")
        req = ParsedRequest()
        req.args = MultiDict([("filter", '[{"term": {"service.code": "i"}}]')])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 2)
        self.assertEqual(resp.docs[0].get("anpa_category")[0].get("qcode"), "i")
        self.assertEqual(resp.docs[1].get("anpa_category")[0].get("qcode"), "i")

        req = ParsedRequest()
        req.args = MultiDict([("service", "i")])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 2)
        self.assertEqual(resp.docs[0].get("anpa_category")[0].get("qcode"), "i")
        self.assertEqual(resp.docs[1].get("anpa_category")[0].get("qcode"), "i")

        req = ParsedRequest()
        req.args = MultiDict([("service", '["a"]')])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 1)
        self.assertEqual(resp.docs[0].get("anpa_category")[0].get("qcode"), "a")

        req = ParsedRequest()
        req.args = MultiDict([("service", "i"), ("subscribers", "sub1")])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 1)
        self.assertEqual(resp.docs[0].get("anpa_category")[0].get("qcode"), "i")
        self.assertEqual(resp.docs[0].get("subscribers")[0], "sub1")

        req = ParsedRequest()
        req.args = MultiDict([("subscribers", "sub2")])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.count(), 2)
        self.assertEqual(resp.docs[0].get("subscribers")[0], "sub2")
        self.assertEqual(resp.docs[1].get("subscribers")[0], "sub2")

    def test_search_capi_aggregations(self):
        self.content_api.publish(
            {
                "_id": "1",
                "guid": "1",
                "type": "text",
                "anpa_category": [{"qcode": "i", "name": "International News"}],
                "headline": "Man bites dog",
                "source": "AAA",
                "urgency": 1,
            },
            [],
        )
        self.content_api.publish(
            {
                "_id": "2",
                "guid": "2",
                "type": "text",
                "anpa_category": [{"qcode": "i", "name": "International News"}],
                "headline": "Man bites cat",
                "source": "BBB",
                "urgency": 2,
            },
            [],
        )

        test = superdesk.get_resource_service("search_capi")
        req = ParsedRequest()
        req.args = MultiDict([("aggregations", 1)])
        resp = test.get(req=req, lookup=None)
        self.assertEqual(resp.hits["aggregations"]["category"]["buckets"][0]["doc_count"], 2)

    def test_associated_item_filter_by_subscriber(self):
        item = {
            "guid": "foo",
            "type": "text",
            "task": {"desk": "foo"},
            "associations": {"featuremedia": {"guid": "a1", "type": "picture", "subscribers": ["sub1"]}},
        }
        subscriber1 = {"_id": "sub1"}
        subscriber2 = {"_id": "sub2"}
        self.content_api.publish(item, [subscriber1, subscriber2])
        self.assertEqual(1, self.db.items.count())
        with self.capi.test_client() as c:
            response = c.get("items/foo", headers=self._auth_headers(subscriber1))
            data = json.loads(response.data)
            self.assertIn("items/foo", data["uri"])
            self.assertEqual(data["associations"]["featuremedia"]["guid"], "a1")

            response = c.get("items/foo", headers=self._auth_headers(subscriber2))
            data = json.loads(response.data)
            self.assertIn("items/foo", data["uri"])
            self.assertNotIn("featuremedia", data["associations"])

    def test_publish_item_with_attachments(self):
        media = io.BytesIO(b"content")
        data = {"media": (media, "media.txt")}
        attachment = {"title": "Test", "description": "test"}
        with self.app.test_request_context("attachments", method="POST", data=data):
            attachment["media"] = request.files["media"]
            store_media_files(attachment, "attachments")  # this would happen automatically otherwise
            superdesk.get_resource_service("attachments").post([attachment])
        self.assertIn("_id", attachment)
        self.assertIsInstance(attachment["media"], ObjectId)

        item = {
            "guid": "foo",
            "type": "text",
            "attachments": [{"attachment": attachment["_id"]}],
            "body_html": '<p><a data-attachment="{}">download</a></p>'.format(str(attachment["_id"])),
        }

        subscriber = {"_id": "sub"}
        self.content_api.publish(item, [subscriber])
        with self.capi.test_client() as c:
            response = c.get("items/foo", headers=self._auth_headers(subscriber))
            data = json.loads(response.data)

        self.assertIn("attachments", data)
        attachments = data["attachments"]
        self.assertEqual(1, len(attachments))
        self.assertEqual("Test", attachments[0]["title"])
        self.assertEqual("test", attachments[0]["description"])
        self.assertEqual("media.txt", attachments[0]["filename"])
        self.assertEqual("text/plain", attachments[0]["mimetype"])
        self.assertEqual(7, attachments[0]["length"])
        self.assertIn("href", attachments[0])
        self.assertIn("media", attachments[0])
        self.assertIn('data-attachment="{}"'.format(attachments[0]["id"]), data["body_html"])

        with self.capi.test_client() as c:
            response = c.get(attachments[0]["href"], headers=self._auth_headers(subscriber))
            self.assertEqual(200, response.status_code, attachments[0]["href"])

    def test_publish_item_with_internal_attachments(self):
        media = io.BytesIO(b"content")
        data = {"media": (media, "media.txt")}
        internal_attachment = {"title": "Test Internal", "description": "test", "internal": True}
        public_attachment = {"title": "Test", "description": "test", "internal": False}
        with self.app.test_request_context("attachments", method="POST", data=data):
            internal_attachment["media"] = request.files["media"]
            public_attachment["media"] = request.files["media"]
            store_media_files(internal_attachment, "attachments")
            store_media_files(public_attachment, "attachments")
            superdesk.get_resource_service("attachments").post([internal_attachment, public_attachment])
        self.assertIn("_id", internal_attachment)
        self.assertIn("_id", public_attachment)
        self.assertIsInstance(internal_attachment["media"], ObjectId)
        self.assertIsInstance(public_attachment["media"], ObjectId)

        item = {
            "guid": "foo-internal",
            "type": "text",
            "attachments": [{"attachment": internal_attachment["_id"]}, {"attachment": public_attachment["_id"]}],
            "body_html": "<p>Foo Bar</p>",
        }

        subscriber = {"_id": "sub"}
        self.content_api.publish(item, [subscriber])
        with self.capi.test_client() as c:
            response = c.get("items/foo-internal", headers=self._auth_headers(subscriber))
            data = json.loads(response.data)

        self.assertIn("attachments", data)
        attachments = data["attachments"]
        # there should be only one attachment (public one)
        self.assertEqual(1, len(attachments))
        self.assertEqual(str(public_attachment["_id"]), attachments[0]["id"])

    def test_items_default_sorting(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        self.content_api.publish(
            {"_id": "aaaa", "guid": "aaa", "urgency": "1", "type": "text", "source": "foo"}, [subscriber]
        )
        # We want to test a default sorting and default sort field is versioncreated,
        # so if we `self.content_api.publish` without delay, `versioncreated` will be almost the same, because it
        # filled automatically in publish service.
        # The lowest dimension is a `second` in datetime format for `versioncreated`.
        # That's why 1 sec delay is used.
        time.sleep(1)
        self.content_api.publish(
            {"_id": "bbbb", "guid": "bbb", "urgency": "2", "type": "text", "source": "foo"}, [subscriber]
        )
        time.sleep(1)
        self.content_api.publish(
            {"_id": "cccc", "guid": "ccc", "urgency": "3", "type": "text", "source": "foo"}, [subscriber]
        )
        time.sleep(1)
        self.content_api.publish(
            {"_id": "dddd", "guid": "ddd", "urgency": "4", "type": "text", "source": "bar"}, [subscriber]
        )
        time.sleep(1)
        self.content_api.publish(
            {"_id": "eeee", "guid": "eee", "urgency": "5", "type": "text", "source": "foo"}, [subscriber]
        )

        with self.capi.test_client() as c:
            # no filters
            response = c.get("items", headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["5", "4", "3", "2", "1"])

            # with filtering
            response = c.get('items?item_source=["foo"]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["5", "3", "2", "1"])

            # with filtering and custom sorting
            response = c.get('items?item_source=["foo"]&sort=[("versioncreated",1)]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["1", "2", "3", "5"])

    def test_items_custom_sorting(self):
        subscriber = {"_id": "sub1"}
        headers = self._auth_headers(subscriber)

        self.content_api.publish(
            {"_id": "aaaa", "guid": "aaa", "urgency": "1", "type": "text", "source": "foo"}, [subscriber]
        )
        self.content_api.publish(
            {"_id": "bbbb", "guid": "bbb", "urgency": "2", "type": "text", "source": "foo"}, [subscriber]
        )
        self.content_api.publish(
            {"_id": "cccc", "guid": "ccc", "urgency": "3", "type": "text", "source": "foo"}, [subscriber]
        )
        self.content_api.publish(
            {"_id": "dddd", "guid": "ddd", "urgency": "4", "type": "text", "source": "bar"}, [subscriber]
        )
        self.content_api.publish(
            {"_id": "eeee", "guid": "eee", "urgency": "5", "type": "text", "source": "foo"}, [subscriber]
        )

        with self.capi.test_client() as c:
            # urgency desc
            response = c.get('items?sort=[("urgency",-1)]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["5", "4", "3", "2", "1"])

            # urgency asc
            response = c.get('items?sort=[("urgency",1)]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["1", "2", "3", "4", "5"])

            # urgency asc + filter
            response = c.get('items?sort=[("urgency",1)]&item_source=["foo"]', headers=headers)
            self.assertEqual(200, response.status_code)
            data = json.loads(response.data)
            self.assertListEqual([i["urgency"] for i in data["_items"]], ["1", "2", "3", "5"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from decimal import Decimal
from . import unit_base
from flask_babel import lazy_gettext

CUBIC_METER_SYMBOL = "cubic meter"
CUBIC_CENTIMETER_SYMBOL = "cubic centimeter"


def convert(cubic_inches, precision=1):
    """
    Converts from cubic inch to cubic m or cubic cm
    :param cubic_inches: Cubic inch value in string
    :return: cubic m/cm value in string, and the symbol
    """
    ci_to_ccm_rate = Decimal(16.3871)
    symbol = CUBIC_CENTIMETER_SYMBOL
    cubic_inches_list = cubic_inches.split("-")
    cubic_centimeter_list = [Decimal(a) * ci_to_ccm_rate for a in cubic_inches_list]

    if any(s for s in cubic_centimeter_list if s > Decimal(100000)):
        # if the value is greater than 100000 then convert it to cubic meters
        cubic_centimeter_list = [
            unit_base.format_converted((s / Decimal(1000000)), precision=1) for s in cubic_centimeter_list
        ]
        symbol = CUBIC_METER_SYMBOL
    else:
        cubic_centimeter_list = [unit_base.format_converted(s, precision=precision) for s in cubic_centimeter_list]

    return "-".join(cubic_centimeter_list), symbol


def cubic_inches_to_metric(item, **kwargs):
    """Converts cubic inch values to metric"""

    regex = r"(\d+-?,?\.?\d*)((\s*)|(-))((cu\.?\s*-?in)|(cb\.?\s*-?in)|([cC]ubic\s*-?([iI]nches|[iI]nch|in)))\b"
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = "cubic_inches_to_metric"
label = lazy_gettext("Volume cubic inches to metric")
callback = cubic_inches_to_metric
access_type = "frontend"
action_type = "interactive"
group = lazy_gettext("volume")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk IO"""

import logging

import superdesk
from superdesk.celery_app import celery
from superdesk.io.ingest import IngestResource, IngestService
from superdesk.io.registry import registered_feed_parsers, allowed_feed_parsers  # noqa
from superdesk.io.registry import registered_feeding_services, allowed_feeding_services  # noqa
from superdesk.io.registry import feeding_service_errors, publish_errors  # noqa
from superdesk.io.registry import FeedParserAllowedResource, FeedParserAllowedService
from superdesk.io.registry import FeedingServiceAllowedResource, FeedingServiceAllowedService
from superdesk.io.registry import get_feeding_service  # noqa

from superdesk.io.commands.add_provider import AddProvider  # noqa
from superdesk.io import importers  # noqa
from superdesk.io.commands.update_ingest import UpdateIngest, update_provider  # noqa
from superdesk.io.commands.remove_expired_content import RemoveExpiredContent
from superdesk.io.ingest_provider_model import IngestProviderResource, IngestProviderService


logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "ingest_providers"
    service = IngestProviderService(endpoint_name, backend=superdesk.get_backend())
    IngestProviderResource(endpoint_name, app=app, service=service)

    from .io_errors import IOErrorsService, IOErrorsResource

    endpoint_name = "io_errors"
    service = IOErrorsService(endpoint_name, backend=superdesk.get_backend())
    IOErrorsResource(endpoint_name, app=app, service=service)

    endpoint_name = "ingest"
    service = IngestService(endpoint_name, backend=superdesk.get_backend())
    IngestResource(endpoint_name, app=app, service=service)

    superdesk.register_resource(
        name="feed_parsers_allowed", resource=FeedParserAllowedResource, service=FeedParserAllowedService
    )
    superdesk.privilege(
        name="feed_parsers", label="Ingest Feed Parsers", description="User can maintain Ingest Feed Parsers."
    )

    superdesk.register_resource(
        name="feeding_services_allowed", resource=FeedingServiceAllowedResource, service=FeedingServiceAllowedService
    )

    superdesk.privilege(
        name="feeding_services", label="Ingest Feed Services", description="User can maintain Ingest Feed Services."
    )


superdesk.privilege(name="ingest_providers", label="Ingest Channels", description="User can maintain Ingest Channels.")


@celery.task(soft_time_limit=15)
def update_ingest():
    """Check ingest providers and trigger an update when appropriate."""
    UpdateIngest().run()


@celery.task(soft_time_limit=600)
def gc_ingest():
    RemoveExpiredContent().run()
